#include "FlowSumCalc.h"
#include <math.h>
#include <fstream>



void FlowSumCalc::addfolderFlowValues(Inputs *input, DataFolder *folder, int timeStep)
{
	if (timeStep == 0)
	{
		input->BQi_NR.insert(input->BQi_NR.end(), input->SimulationNumericalParams["TotalTimeSteps"], 0.0);
		input->PAQi_NR.insert(input->PAQi_NR.end(), input->SimulationNumericalParams["TotalTimeSteps"], 0.0);
		input->DCIAQi_NR.insert(input->DCIAQi_NR.end(), input->SimulationNumericalParams["TotalTimeSteps"], 0.0);	
	}
	double fracArea = folder->ParamDict["Area"]/ input->SimulationNumericalParams["CatchmentArea_m2"];
	double fPervious_total = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] + folder->ParamDict["SoilCover_frac"];
	double fPervious_in_folder = 0.0;
	if (fracArea != 0.0)
		fPervious_in_folder = (fPervious_total / fracArea);
	else
		fPervious_in_folder = 0.0;

	double folderBQ = folder->VarDict["baseFlow"] * fPervious_in_folder;
	
	double SatExQ = folder->VarDict["satExRunoff"];
	double infilExQ = folder->VarDict["infilExRunoff"] * folder->ParamDict["InfiltExcessGovernedArea_frac"];
	
	double folderPAQ = (SatExQ + infilExQ) * fPervious_in_folder;
	double folderDCIAQ = folder->VarDict["impRunoffToOutlet_m"];

	input->BQi_NR[timeStep] = input->BQi_NR[timeStep] + (folderBQ/input->LocationParams["NumberOfLocations"]);
	input->PAQi_NR[timeStep] = input->PAQi_NR[timeStep] + (folderPAQ / input->LocationParams["NumberOfLocations"]);
	input->DCIAQi_NR[timeStep] = input->DCIAQi_NR[timeStep] + (folderDCIAQ / input->LocationParams["NumberOfLocations"]);

}

void FlowSumCalc::calculateSpatial(Inputs *input, DataFolder *folder, int timeStep)
{


	double area;              // area this folder makes up
	double fArea;             // fraction of the watershed this folder makes up
	double SatExQ;
	double infilExQ;
	double BQ;
	double PAQ;
	double PAQ_unAdj;
	double DCIAQ;
	double totalQ;
	double UZQ;                              // unsaturated zone flow for a folder

	double sumBQ = 0.0;
	double sumPAQ = 0.0;
	double sumDCIAQ = 0.0;
	double sumTotalQ = 0.0;
	double sumUZQ = 0.0;                     // total unsaturated zone flow for a folder

	double watershedArea = input->SimulationNumericalParams["CatchmentArea_m2"];    // area of the entire watershed

	area = 0.0;              // area this folder makes up
	fArea = 0.0;             // fraction of the watershed this folder makes up
	SatExQ = 0.0;
	infilExQ = 0.0;
	BQ = 0.0;
	PAQ = 0.0;
	DCIAQ = 0.0;
	totalQ = 0.0;
	UZQ = 0.0;

	double fPervious_total = 0.0;
	double fPervious_in_folder = 0.0;

	fPervious_total = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] + folder->ParamDict["SoilCover_frac"];
	double fracArea = folder->ParamDict["Area"] / input->SimulationNumericalParams["CatchmentArea_m2"];
	// Check for BulkArea folder or Tree Pits that covers situations where their folder percent (and area) == 0
	if (fracArea != 0.0)
		fPervious_in_folder = (fPervious_total / fracArea);
	else
		fPervious_in_folder = 0.0;

	//if (folder->ParamStringDict["Type"]  == "BulkArea" || folder->ParamStringDict["Type"]  == "TreePit")
	if (folder->ParamStringDict["Type"] !="RainBarrel")
	{

		// THE SATURATION EXCESS FLOW PERCENTAGE ONLY GOVERNS INFILTRATION, NOT RUNOFF GENERATION
		SatExQ = folder->VarDict["satExRunoff"];

		// saturation excess flow = saturation excess flow at this timestep (for the folder) * (1 - fraction of watershed with infiltration excess)
		//SatExQ = folder->VarDict["satExRunoff"] * (1-folder->ParamDict["InfiltExcessGovernedArea_frac"]);     
		// infiltration excess flow = infiltration excess flow at this timestep (for the folder) * fraction of watershed with infiltration excess
		infilExQ = folder->VarDict["infilExRunoff"] * folder->ParamDict["InfiltExcessGovernedArea_frac"];

		
		// Base Flow = Saturated zone base flow * fraction of the watershed that is pervious * fraction of the total watershed we are dealing with (if dealing in sub-watershed)
		BQ = folder->VarDict["baseFlow"] * fPervious_in_folder;
		// Pervious Flow  = (fraction o f the watershed that  is pervious * fraction of the watershed we are dealing with * (Saturation excess flow + Infiltration excess flow))
		PAQ_unAdj = (SatExQ + infilExQ);
		PAQ = (SatExQ + infilExQ) * fPervious_in_folder;
		// Impervious Flow = Impervious Runoff * fraction of the watershed we are dealing with
		// Impervious runoff (to DCIA) has already been multiplied by the appropriate fraction
		DCIAQ = folder->VarDict["impRunoffToOutlet_m"];

		// Unsaturated zone flow - does this need to be times by pervious area?  
		// Water balance needs ot sort this out - right now assuming yes
		UZQ = folder->VarDict["unsatZoneQ"] * fPervious_in_folder;

	}
	else
	{

		BQ = folder->VarDict["baseFlow"];
		PAQ = folder->VarDict["perRunoff"];
		DCIAQ = folder->VarDict["impRunoffToOutlet_m"];
		UZQ = folder->VarDict["unsatZoneQ"];
	}

	// getting the total flow
	totalQ = BQ + PAQ + DCIAQ;

	// storing these timestep values
	folder->VarDict["baseFlow"]=BQ;
	//folder->VarVecDict["perAreaFlow_unAdj"].emplace_back(PAQ_unAdj);
	folder->VarDict["perAreaFlow"]=PAQ;
	folder->VarDict["impAreaFlow"]=DCIAQ;
	//folder->VarVecDict["totalFlow"].emplace_back(totalQ);


}


// This calcualtion goes through and sums the predicted runoff/streamflow from each folder within the semi-distributed organizer

void FlowSumCalc::calculateStatistical(Inputs *input, std::vector<DataFolder*> &stafolders, int timeStep)
{

	double area;              // area this folder makes up
	double fArea;             // fraction of the watershed this folder makes up
	double SatExQ;
	double infilExQ;
	double BQ;
	double PAQ;
	double PAQ_unAdj;
	double DCIAQ;
	double totalQ;
	double UZQ;                              // unsaturated zone flow for a folder

	double sumBQ = 0.0;
	double sumPAQ = 0.0;
	double sumDCIAQ = 0.0;
	double sumTotalQ = 0.0;
	double sumUZQ = 0.0;                     // total unsaturated zone flow for a folder

	// GI Rain barrel variables - RC, 5/17 
	double waterOnRainBarrelArea = 0.0;
	double rainBarrelFlowToInfil = 0.0;
	double rbStorage = 0.0;
	double rainBarrelStorageMax = 0.0;

	double watershedArea = input->SimulationNumericalParams["CatchmentArea_m2"];    // area of the entire watershed


	// Stepping through and calculating the runoff/streamflow of each folder in the watershed
	for(int i = 0; i < stafolders.size(); ++i)
	{
		DataFolder* folder = stafolders[i];

		area = 0.0;              // area this folder makes up
		fArea = 0.0;             // fraction of the watershed this folder makes up
		SatExQ = 0.0;
		infilExQ = 0.0;
		BQ = 0.0;
		PAQ = 0.0;
		DCIAQ = 0.0;
		totalQ = 0.0;
		UZQ = 0.0;

		double fPervious_total = 0.0;
		double fPervious_in_folder = 0.0;

		fPervious_total = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] +  folder->ParamDict["ShortVegetationCover_frac"] + folder->ParamDict["SoilCover_frac"];
		double fracArea = folder->ParamDict["Area"] / input->SimulationNumericalParams["CatchmentArea_m2"];
		// Check for BulkArea folder or Tree Pits that covers situations where their folder percent (and area) == 0
		if (fracArea != 0.0)
			fPervious_in_folder = (fPervious_total / fracArea);
		else
			fPervious_in_folder = 0.0;

		//if (folder->ParamStringDict["Type"]  == "BulkArea" || folder->ParamStringDict["Type"]  == "TreePit")
		if(folder->ParamStringDict["Type"]  !="RainBarrel")
		{

			// THE SATURATION EXCESS FLOW PERCENTAGE ONLY GOVERNS INFILTRATION, NOT RUNOFF GENERATION
			SatExQ = folder->VarDict["satExRunoff"];     

			// saturation excess flow = saturation excess flow at this timestep (for the folder) * (1 - fraction of watershed with infiltration excess)
			//SatExQ = folder->VarDict["satExRunoff"] * (1-input->paramValues->getParameter("InfiltExcessGovernedArea_frac"));     
			// infiltration excess flow = infiltration excess flow at this timestep (for the folder) * fraction of watershed with infiltration excess

			infilExQ = folder->VarDict["infilExRunoff"]* folder->ParamDict["InfiltExcessGovernedArea_frac"];
			// Base Flow = Saturated zone base flow * fraction of the watershed that is pervious * fraction of the total watershed we are dealing with (if dealing in sub-watershed)
			BQ = folder->VarDict["baseFlow"] * fPervious_in_folder;
			// Pervious Flow  = (fraction o f the watershed that  is pervious * fraction of the watershed we are dealing with * (Saturation excess flow + Infiltration excess flow))
			PAQ_unAdj = (SatExQ + infilExQ);
			PAQ = (SatExQ + infilExQ) * fPervious_in_folder;

			// Impervious Flow = Impervious Runoff * fraction of the watershed we are dealing with
			// Impervious runoff (to DCIA) has already been multiplied by the appropriate fraction
			DCIAQ = folder->VarDict["impRunoffToOutlet_m"];		
			
			// Reza, for the GI output as the pervious and impervious flows affected by the GI
			if (folder->ParamStringDict["Type"] == "BulkArea") {
				input->RepoDict["PAQ"] = PAQ;
				input->RepoDict["DCIAQ"] = DCIAQ;
				input->RepoDict["AveSMD_BA"] = folder->VarVecDict["rootZoneStorDef_TI"][24];
			}

			// Unsaturated zone flow - does this need to be times by pervious area?  
			// Water balance needs ot sort this out - right now assuming yes
			UZQ = folder->VarDict["unsatZoneQ"] * fPervious_in_folder;

		} 
		else  
		{
			// GI Rain barrel specific variables - RC, 5/17
			waterOnRainBarrelArea = folder->VarDict["waterOnRainBarrelArea"];
			rainBarrelFlowToInfil = folder->VarDict["rainBarrelFlowToInfil"];
			rbStorage = folder->VarDict["rbStorage"];
			rainBarrelStorageMax = folder->VarDict["rainBarrelStorageMax"];

			BQ = folder->VarDict["baseFlow"];
			PAQ = folder->VarDict["perRunoff"];
			DCIAQ = folder->VarDict["impRunoffToOutlet_m"];
			UZQ = folder->VarDict["unsatZoneQ"];
		}

		// getting the total flow
		totalQ = BQ + PAQ + DCIAQ;

		// storing these timestep values
		folder->VarVecDict["baseFlow"].emplace_back(BQ);
		folder->VarVecDict["perAreaFlow_unAdj"].emplace_back(PAQ_unAdj);
		folder->VarVecDict["perAreaFlow"].emplace_back(PAQ);
		folder->VarVecDict["impAreaFlow"].emplace_back(DCIAQ);
		folder->VarVecDict["totalFlow"].emplace_back(totalQ);
		
		// GI Rain Barrel specific timestep values - RC, 5/17
		folder->VarVecDict["waterOnRainBarrelArea"].emplace_back(waterOnRainBarrelArea);
		folder->VarVecDict["rainBarrelFlowToInfil"].emplace_back(rainBarrelFlowToInfil);
		folder->VarVecDict["rbStorage"].emplace_back(rbStorage);

		// Adjusting by the fraction of the watershed that each folder represents
		area = folder->ParamDict["Area"];
		fArea = area / watershedArea;
		BQ *= fArea;
		PAQ *= fArea;
		DCIAQ *= fArea;
		totalQ *= fArea;
		UZQ *= fArea;

		// storing these area adjusted timestep values
		folder->VarVecDict["adjBaseFlow"].emplace_back(BQ);
		folder->VarVecDict["adjPerAreaFlow"].emplace_back(PAQ);
		folder->VarVecDict["adjImpAreaFlow"].emplace_back(DCIAQ);
		folder->VarVecDict["adjTotalFlow"].emplace_back(totalQ);

		// adding the adjusted folder value to the watershed total 
		sumBQ += BQ;
		sumPAQ += PAQ;
		sumDCIAQ += DCIAQ;
		sumTotalQ += totalQ;
		sumUZQ += UZQ;
	}



	// setting the non-time distributed vector values
	// these values are later distributed in time by the FlowRouteCalc
	input->BQi_NR.push_back(sumBQ);
	//repo->runoff.setBQi_NR(sumBQ, timeStep);
	input->PAQi_NR.push_back(sumPAQ);
	//repo->runoff.setPAQi_NR(sumPAQ, timeStep);
	input->DCIAQi_NR.push_back(sumDCIAQ);
	//repo->runoff.setDCIAQi_NR(sumDCIAQ, timeStep);
	//input->RepoVecDict["TotalQi_NR"].push_back(sumTotalQ);
	//repo->runoff.setTotalQi_NR(sumTotalQ, timeStep);
	//input->RepoVecDict["TotalUnsatZoneQ_TS"].push_back(sumUZQ);
	//repo->unsaturatedZone.setTotalUnsatZoneQ_TS(sumUZQ, timeStep);


}