#include "ImperviousDepressionStorageCalc.h"

void ImperviousDepressionStorageCalc::calculateImpEvapTemperature(Inputs *input, DataFolder *folder, int timeStep)
{
#define lamda 2257000.0 //cte give units
	//ImperviousLE > 0 when water is available, determined in HeatFluxCal::StateCal wateronImp = folder->VarDict["impDepStor_m"] / UrbanDepression;
	folder->VarDict["impEvap_m"] = folder->VarDict["ImperviousLE"] / lamda;
}

void ImperviousDepressionStorageCalc::calculateImpEvapStatistical(Inputs *input, DataFolder *folder, int timeStep)
{
	if (folder->VarDict["impDepStor_m"] > 0.0) {
		folder->VarDict["impEvap_m"] = (folder->VarDict["impDepStor_m"] / folder->VarDict["impDepStorMax_m"]) * input->PE[timeStep];
	}
	else
		folder->VarDict["impEvap_m"] = 0.0;
}

void ImperviousDepressionStorageCalc::calculate(Inputs *input, DataFolder *folder, int timeStep, std::map<std::string, void(*)(Inputs *input, DataFolder *folder, int timeStep)> &functors)
{
	// impDepStorMax = param.dat impervious storage maximum value (in mm) * 0.001 -> to convert to meters
	//cte Feb 17, 2020 ImperviousDepressionStorage_mm should be getting different values for SpatialTemperatureHydro based on Land Cover, e.g., Urban21depression
	folder->VarDict["impDepStorMax_m"] = 0.001 * folder->ParamDict["ImperviousDepressionStorage_mm"];

	// Impervious Depression Storage Previous = impDepStor  (Shifts the current impervious depression storage to the temp variable)
	double impDepStorPrev_m = folder->VarDict["impDepStor_m"];
	double impPrecip_m = 0.0;
	// increment in pervious Storage
	double incrImpDepStor = 0.0; 
    impPrecip_m = folder->VarDict["waterOnImperviousArea"];
	if (folder->VarDict["snowOpen"] <= 0.0)
	{
		functors["calculateImpEvap"](input, folder, timeStep);
	}
	else {
		// Otherwise set CurrentEimp (0) - the water won't evaporate if there is snow on the ground.
		// and we must protect against divide by 0 errors
		folder->VarDict["impEvap_m"] = 0.0;
	}

	// cte, Feb 17 2020, not sure who added this next comment and if 2 lines of code is needed
	// Below added to fix undefined values when StoragePervious = 0.0
	if (folder->ParamDict["ImperviousDepressionStorage_mm"] <= 0.0)
		folder->VarDict["impEvap_m"] = 0.0;


	// Modify the (current)  Impervious Depression Storage = Impervious Depression Storage + Precip_mpdt - Evap
	folder->VarDict["impDepStor_m"] = folder->VarDict["impDepStor_m"] + impPrecip_m - folder->VarDict["impEvap_m"]; // update interception storage

	         
	// If the UrStorageTime is < ImperviousStorageMax
	if(folder->VarDict["impDepStor_m"] < folder->VarDict["impDepStorMax_m"])
	{
			folder->VarDict["impRunoffToOutlet_m"] = 0.0;          // If the amount stored is less than the max, there is no Impervious runoff
			folder->VarDict["impRunoffToSoil_m"] = 0.0;     // If the amount stored is less than the max, ther is no Impervious flow to soil
	}
	// If the UrStorageTime is >= ImperviousStorageMax
	else if(folder->VarDict["impDepStor_m"] >= folder->VarDict["impDepStorMax_m"])
	{
		// If the previous UrStorage < Storage Max   (meaning the amount that put it over the max has occurred in this timestep)
		if (impDepStorPrev_m < folder->VarDict["impDepStorMax_m"])
		{										
			// CurrentUrQ = (ImperviousPrecip - (ImperviousStorageMax - Previous Impervious Storage) - CurrentImperviousEvaporation) * fraction of connected impervious area(DCIA))
			// CurrentUrQ = ((Increase in Impervious area water - amount evaporated) - (the water available in Impervious storage) * DCIA) = Impervious flow
			folder->VarDict["impRunoffToOutlet_m"] = (impPrecip_m - (folder->VarDict["impDepStorMax_m"] - impDepStorPrev_m) - folder->VarDict["impEvap_m"]) * folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"];
			// CurrentUrQ = (ImperviousPrecip - (ImperviousStorageMax - Previous Impervious Storage) - CurrentImperviousEvaporation) * fraction of disconnected impervious area(1-DCIA))
			// CurrentUrQ = ((Increase in Impervious area water - amount evaporated) - (the water available in Impervious storage) * disconnected impervious area (1-DCIA)) = Impervious flow to soil
			folder->VarDict["impRunoffToSoil_m"] = (impPrecip_m - (folder->VarDict["impDepStorMax_m"] - impDepStorPrev_m) - folder->VarDict["impEvap_m"]) * folder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"];
		}

		else
		{
			// This Else statement covers the situation of ImperviousStorageTime1 being equal to the storage max
			// In this case, the ImperviousPrecip is split into the amount heading to connected impervious area (DCIA) and the amount heading to disconnected impervious area (1-DCIA)
			folder->VarDict["impRunoffToOutlet_m"] = (impPrecip_m - folder->VarDict["impEvap_m"]) * folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"];
			folder->VarDict["impRunoffToSoil_m"] = (impPrecip_m - folder->VarDict["impEvap_m"]) * folder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"];
		}
	    // Either of the two preceding cases leaves the current Impervious Storage equal to the storage max 
		folder->VarDict["impDepStor_m"] = folder->VarDict["impDepStorMax_m"];
	}

	//// GI specific function
	//if (folder->ParamStringDict["Type"] != "BulkArea") {
	//	GI_ImpCalc(input, folder, timeStep);
	//}

	// Making sure that impervious runoff on the DCIA isn't < 0
	if(folder->VarDict["impRunoffToOutlet_m"] < 0.0)
		folder->VarDict["impRunoffToOutlet_m"] = 0.0;

	// Making sure that impervious runoff on the (1-DCIA) isn't < 0
	if(folder->VarDict["impRunoffToSoil_m"] < 0.0)
		folder->VarDict["impRunoffToSoil_m"] = 0.0;

	// Making sure that the amount currently stored in impervious depression storage isn't < 0
	if (folder->VarDict["impDepStor_m"] < 0.0)
		folder->VarDict["impDepStor_m"] = 0.0;


	//Reza: Here we calculate the effective impervious runoff in the contributing area as a function of the "impRunoffDCIA"
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		input->RepoDict["impDCIA"] = folder->VarDict["impRunoffToOutlet_m"];
	}
	if (folder->ParamStringDict["Type"] != "BulkArea") {
		//input->RepoDict["impRunoffContArea"] = input->RepoDict["impDCIA"] *
		//	((folder->ParamDict["upSlopeArea"] * folder->ParamDict["upSlopeAreaImpFrac"] *
		//		folder->ParamDict["ImpAreaTreated_frac"]) /
		//		(input->SimulationNumericalParams["CatchmentArea_m2"] - (folder->ParamDict["Area"] * folder->ParamDict["giNumber"])));
		
		// This is not multipyed by folder->ParamDict["upSlopeAreaImpFrac"] b/c impDCIA is already adjusted
		input->RepoDict["impRunoffContArea"] = input->RepoDict["impDCIA"] *
			((folder->ParamDict["upSlopeArea"] * folder->ParamDict["ImpAreaTreated_frac"]));// /
		//		(input->SimulationNumericalParams["CatchmentArea_m2"] - (folder->ParamDict["Area"] * folder->ParamDict["giNumber"])));
		

		// To convert the available water for the GI from the cont. area to the GI area
		
		input->RepoDict["impRunoffContArea_GI"] = (input->RepoDict["impRunoffContArea"] ) /
			(folder->ParamDict["Area"] * folder->ParamDict["giNumber"]);
		//folder->VarDict["waterToPerDep"] += input->RepoDict["impRunoffContArea"];
	}
	//if (folder->ParamStringDict["Type"] == "BulkArea") {
	//	folder->VarDict["impRunoffToOutlet_m"] -= input->RepoDict["impRunoffContArea"];
	//	if (folder->VarDict["impRunoffToOutlet_m"] < 0.0) {
	//		folder->VarDict["impRunoffToOutlet_m"] = 0.0;
	//	}
	//}

	storeValues(input,folder, timeStep);

}

// With the routed runoff, we have to be careful not to double count!  The runoff routed to a folder must be split between the runoff routed
// to impervious area and the runoff routed to pervious area.  And then some of the runoff routed to impervious area will be routed to 
// pervious area since on a fraction of the impervious area (based on the DCIA) is routed directly to the channel?
// But can we just added the weighted impervious area water/precip amount to the runoff amount, or does a weighting of the runoff amount also have to occur?



void ImperviousDepressionStorageCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	folder->VarDict["impRunoff"] = folder->VarDict["impRunoffToOutlet_m"] + folder->VarDict["impRunoffToSoil_m"];
	folder->VarDict["totalImpEvap"] += folder->VarDict["impEvap_m"];
	folder->VarDict["totalImpRunoffDCIA"] += folder->VarDict["impRunoffToOutlet_m"];
	folder->VarDict["totalImpRunoffToSoil"] += folder->VarDict["impRunoffToSoil_m"];
	folder->VarDict["totalImpRunoff"] += folder->VarDict["impRunoff"];
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		// Updating vector variables
		folder->VarVecDict["impDepEvap"].emplace_back(folder->VarDict["impEvap_m"]);
		folder->VarVecDict["impDepStor"].emplace_back(folder->VarDict["impDepStor_m"]);
		folder->VarVecDict["impRunoff"].emplace_back(folder->VarDict["impRunoff"]);
		folder->VarVecDict["impRunoffDCIA"].emplace_back(folder->VarDict["impRunoffToOutlet_m"]);
		folder->VarVecDict["impRunoffToSoil"].emplace_back(folder->VarDict["impRunoffToSoil_m"]);
	}
}





