#ifndef INFIL_H
#define INFIL_H

//////////////////////////////////////////////////////////////////////////////////
//																		        //
// InfiltrationPowDecay.h - Calculates infitration [K decay with N = 1]                 //																		   
// ver 1.0                                                                      //
// Language:    Visual C++ 2010                                                 //
// Platform:    MacBookPro5,4  Win7										        //
// Application: HydroPrime/iHydro Version6, Summer 2012                         //
// Programmer:  				        //																	   	
//////////////////////////////////////////////////////////////////////////////////
/*
  Origin:
  ==================
  Program: iHydro Version 5 & iHydro Version 4.5
  
  Source Code File(s): sInfiltrationN.cpp, sInfiltration.cpp 
  
  Author(s): Jun Wang, jwang06@mailbox.syr.edu, Original author 
			  	
						*Incorporated calculations into new architecture
  Creation Date: Summer 2000

  Publication(s): Jun Wang PHDThesis

  Module Operations: 
  ==================
  ...

  Public Interface:
  =================
  SoilInfiltrationCalc(Inputs *input, Repo *repo);
  void calculate(Inputs *input, DataFolder *folder, int timeStep);

  Build Process:
  ==============
  Required files
	-ICalc.h, InputsRepository.h, ResultsRepository.h, Infiltration.cpp 
				
  Build commands 
    - devenv hydroprime.sln
   
  Maintenance History:
  ====================

 */

#include "../Inputs/Inputs.h"
#include "../DataFolder.h"

class SoilInfiltrationPowDecayCalc
{
	public:	
		static void calculate(Inputs *input, DataFolder *folder, int timeStep);
	private:
		static void beforePonding(int bIT,  double bR, double bDT, DataFolder *folder, Inputs * input);
		static void estimateTP(int tpIT,  double tpR, double tpDT, DataFolder *folder, Inputs * input);
		static void inPonding(int ipIT, double ipPP, double ipDT, DataFolder *folder, Inputs *input);
		static void simpsonIntegration(double IP, double F, double& IR, DataFolder *folder, Inputs *input);
		static void noPonding(double npR, Inputs *input, DataFolder *folder, int timeStep);
		static void infilEXQ(double ifR, Inputs *input, DataFolder *folder, int timeStep);

};



#endif