#include "InputTopographyData.h"




TopographyData::TopographyData(std::string directory, std::string simulationMode, std::string routingAlgorithm, double numfolders, double simulationNoDataVal) :
	directory(directory), simulationMode(simulationMode), routingAlgorithm(routingAlgorithm), numfolders(numfolders), simulationNoDataVal(simulationNoDataVal) {}




double TopographyData::getAveSoilMoistureDeficit(double topAveIndex, double T0, double mM, double Q0, double nN)
{
	// double topAveIndex, double T0, double mM, double Q0, double nN) :
	double aveSMD = 0.0;
	if (simulationMode.find("PowerDecay") != std::string::npos)
	{
		// used for power decay calcualtions
		double SZQ = 0.0;
		//Q0 = 1*pow(10, -5);    // For testing purposes 
		SZQ = pow(Q0 / T0, 1 / nN)*(topAveIndex);

		if (SZQ >1.0)
			SZQ = 1.0;
		if (SZQ <0.0)
			SZQ = 0.0;
		aveSMD = mM * (1.0 - SZQ);
	}
	else
	{
		// used for exponential decay calculations
		double SZQ = 0.0;
		SZQ = T0 * exp(-topAveIndex);
		aveSMD = -mM * log(Q0 / SZQ);

	}
	return aveSMD;
}

void TopographyData::init(double N)
{

	if (simulationMode == "StatisticalHydroPowerDecay")
	{
		//! is negative of, and calcIndexFile is boolean. So, if the calcIndexFile does not work and returns False, negative of False is True, and the code will throw 1 exception. 
		if (!calcIndexFile(directory, N, simulationMode, routingAlgorithm))                  // For semi-distributed power decay
		{
			throw 1;
		}

	}
	else if (simulationMode == "StatisticalHydroExponentialDecay")
	{
		if (!calcIndexFile(directory, simulationMode, routingAlgorithm))                    // For semi-distrbuted exponential decay
		{
			throw 1;
		}
	}
	else if (simulationMode == "LCScalingTempSpatialPowerDecay" || simulationMode == "SpatialTemperatureHydroPowerDecay"|| simulationMode == "CoolRiverPowerDecay")
	{
		if (!calcDistIndexFile(directory, N, simulationMode, routingAlgorithm))                    // For distributed power decay 
		{
			throw 1;
		}
	}
	else if (simulationMode == "LCScalingTempSpatialExponentialDecay" || simulationMode == "SpatialTemperatureHydroExponentialDecay")
	{
		if (!calcDistIndexFile(directory, simulationMode, routingAlgorithm))                    // For destributed exponential decay
		{
			throw 1;
		}
	}
}


bool TopographyData::readSlopeAspectFile()
{
	double temp, NODATA;
	std::string space, line;
	std::ifstream readSlopeFile(directory + "\\Slope.dat");
	if (!readSlopeFile)
	{
		std::cout << "Slope.dat does not exist\n";
		std::cout << "To generate, delete TIorganizer.dat and FDorganizer.dat input files.\n Make sure DEM.dat in input folder.\n Rerun.\n";
		return false;
	}

	// Reading in the header
	int xllcorner, yllcorner;
	getline(readSlopeFile, line);
	istringstream(line) >> space >> ncols;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> NODATA;


	// Reading in the values
	while (readSlopeFile >> temp)
	{
		if (temp != NODATA)
		{
			slope1D.push_back(temp);
		}
		else
		{
			slope1D.push_back(simulationNoDataVal);
		}
	}
	readSlopeFile.close();

	std::ifstream readAspectFile(directory + "\\Aspect.dat");
	if (!readAspectFile)
	{
		std::cout << "Aspect.dat does not exist\n";
		std::cout << "To generate, delete TIorganizer.dat and FDorganizer.dat input files.\n Make sure DEM.dat in input folder.\n Rerun.\n";
		return false;
	}

	// Reading in the header
	getline(readAspectFile, line);
	istringstream(line) >> space >> ncols;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> NODATA;


	// Reading in the values
	while (readAspectFile >> temp)
	{
		if (temp != NODATA)
		{
			aspect1D.push_back(temp);
		}
		else
		{
			aspect1D.push_back(simulationNoDataVal);
		}
	}
	readAspectFile.close();
	return true;
}


bool TopographyData::calcDistIndexFile(std::string &directory, double N, std::string simulationMode, std::string routingAlgorithm)
{
	subFarea = 1.0;
	averageIndex = 0.0;
	indexSize = 0;

	double temp, NODATA;
	std::string space;
	std::string line, temp2;
	// If TIorganizer.dat and FlowDirorganizer.dat do not exist, invest computation time in creating them.
	if (!fileExists(directory + "\\TIorganizer.dat") && !fileExists(directory + "\\FDorganizer.dat"))
	{
		IndexCalculator calc(directory, N, simulationMode, routingAlgorithm);
		if (calc.getParseSucess())
		{
			//calc.createTIFile();
			calc.getElevation1D(elevation1D);
			setcols(calc.getcols());
			setrows(calc.getrows());
			setxllcorner(calc.getxllcorner());
			setyllcorner(calc.getyllcorner());
		}
		else
		{
			//writeToLog.addComment("TIorganizer.dat not be created! Make sure that the dem.dat is the input directory and check its format.  ");
			return false;
		}
	}
	// If TIorganizer.dat and FlowDirorganizer.dat do exist, do not invest computation time in creating them.
	else 
	{
		IndexCalculator calc(directory, N, simulationMode, routingAlgorithm); 
		calc.getElevation1D(elevation1D); 
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////
	// Read in the TIorganizer.dat - must be kept separate from the creation in case it is already created
	std::ifstream readTIorganizer(directory + "\\TIorganizer.dat");
	if (!readTIorganizer)
	{
		//writeToLog.addComment("TIorganizer.dat could not be opened!");
		return false;
	}

	// Reading in the header
	int xllcorner, yllcorner;
	getline(readTIorganizer, line);
	istringstream(line) >> space >> ncols;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> NODATA;


	// Reading in the values
	while (readTIorganizer >> temp)
	{
		if (temp != NODATA)
		{
			index.push_back(temp);
		}
		else
		{
			index.push_back(simulationNoDataVal);
		}
	}
	indexArea.push_back(1.0);
	readTIorganizer.close();
	//////////////////////////////////////////////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////////////////////////////////////////////
	// Reading in the FDorganizer.dat FIle 
	std::ifstream readFDorganizer(directory + "\\FDorganizer.dat");
	if (!readFDorganizer)
	{
		//writeToLog.addComment("FDorganizer.dat could not be opened!");
		return false;
	}

	// Reading in the header
	getline(readFDorganizer, line);
	istringstream(line) >> space >> ncols;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> NODATA;

	// Reading in the values
	while (readFDorganizer >> temp)
	{
		if (temp != NODATA)
		{
			FDorganizer_1D.push_back(temp);
		}
		else
		{
			FDorganizer_1D.push_back(simulationNoDataVal);
		}

	}
	readFDorganizer.close();
	//////////////////////////////////////////////////////////////////////////////////////////////////


	// Function to calculation the proportionVector - vector that stores the proportion of runoff that flows to downstream folders
	calcPropVector();

	if (index.size() < static_cast<size_t>(numfolders))
	{
		std::string strExpected, strActual;
		//writeToLog.convertToString(paramValues.getParameter("NumberOfLocations"), strExpected);
		//writeToLog.convertToString(index.size(), strActual);
		//writeToLog.addComment("There are not enough TIorganizer.dat values for the number of folders specified in the param.dat.");
		//writeToLog.addComment("Expected number: " + strExpected);
		//writeToLog.addComment("Actual number: " + strActual);
		return false;
	}
	else if (!distAveIndex())
		return false;
	else
		return true;
}



bool TopographyData::calcDistIndexFile(std::string &directory, std::string simulationMode, std::string routingAlgorithm)
{
	subFarea = 1.0;
	averageIndex = 0.0;
	indexSize = 0;

	double temp, NODATA;
	std::string space;
	std::string line, temp2;
	// If TIorganizer.dat and FlowDirorganizer.dat do not exist, invest computation time in creating them.
	if (!fileExists(directory + "\\TIorganizer.dat") && !fileExists(directory + "\\FDorganizer.dat"))
	{

		IndexCalculator calc(directory, simulationMode, routingAlgorithm);
		if (calc.getParseSucess())
		{
			//calc.createTIFile();
			calc.getElevation1D(elevation1D);
		}
		else
		{
			//writeToLog.addComment("TIorganizer.dat not be created! Make sure that the dem.dat is the input directory and check its format.  ");
			return false;
		}
	}
	// If TIorganizer.dat and FlowDirorganizer.dat do exist, do not invest computation time in creating them.
	else
	{
		IndexCalculator calc(directory, simulationMode, routingAlgorithm);
		calc.getElevation1D(elevation1D);
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////
	// Read in the TIorganizer.dat - must be kept separate from the creation in case it is already created
	std::ifstream readTIorganizer(directory + "\\TIorganizer.dat");
	if (!readTIorganizer)
	{
		//writeToLog.addComment("TIorganizer.dat could not be opened!");
		return false;
	}

	// Reading in the header
	int xllcorner, yllcorner;
	getline(readTIorganizer, line);
	istringstream(line) >> space >> ncols;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> NODATA;


	// Reading in the values
	while (readTIorganizer >> temp)
	{
		if (temp != NODATA)
		{
			index.push_back(temp);
		}
		else
		{
			index.push_back(simulationNoDataVal);
		}
	}
	indexArea.push_back(1.0);
	readTIorganizer.close();
	//////////////////////////////////////////////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////////////////////////////////////////////
	// Reading in the FDorganizer.dat FIle 
	std::ifstream readFDorganizer(directory + "\\FDorganizer.dat");
	if (!readFDorganizer)
	{
		//writeToLog.addComment("FDorganizer.dat could not be opened!");
		return false;
	}

	// Reading in the header
	getline(readFDorganizer, line);
	istringstream(line) >> space >> ncols;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> NODATA;

	// Reading in the values
	while (readFDorganizer >> temp)
	{
		if (temp != NODATA)
		{
			FDorganizer_1D.push_back(temp);
		}
		else
		{
			FDorganizer_1D.push_back(simulationNoDataVal);
		}

	}
	readFDorganizer.close();
	//////////////////////////////////////////////////////////////////////////////////////////////////


	// Function to calculation the proportionVector - vector that stores the proportion of runoff that flows to downstream folders
	calcPropVector();

	if (index.size() < static_cast<size_t>(numfolders))
	{
		std::string strExpected, strActual;
		//writeToLog.convertToString(paramValues.getParameter("NumberOfLocations"), strExpected);
		//writeToLog.convertToString(index.size(), strActual);
		//writeToLog.addComment("There are not enough TIorganizer.dat values for the number of folders specified in the param.dat.");
		//writeToLog.addComment("Expected number: " + strExpected);
		//writeToLog.addComment("Actual number: " + strActual);
		return false;
	}
	else if (!distAveIndex())
		return false;
	else
		return true;
}



bool TopographyData::distAveIndex()
{
	int cnt = 0;
	averageIndex = 0;
	for (size_t i = 0; i < index.size(); i++)
	{
		if (index[i] != simulationNoDataVal)
		{
			averageIndex = averageIndex + index[i];
			cnt++;
		}
	}

	if (cnt == 0)
	{
		//writeToLog.addComment("Error calcualting distributed average index. \n No index values with data.");
		return false;
	}
	else
	{

		averageIndex = averageIndex / cnt;
		return true;
	}
}




bool TopographyData::fileExists(std::string path)
{
	std::ifstream checkFile(path);

	if (!checkFile)
	{
		return false;
	}
	else
	{
		checkFile.close();
		return true;
	}
}




bool TopographyData::calcIndexFile(std::string &directory, std::string simulationMode, std::string routingAlgorithm)
{
	if (!fileExists(directory + "expdecayTI.dat"))
	{
		IndexCalculator calc(directory, simulationMode, routingAlgorithm);
		if (calc.getParseSucess())
			calc.createIndexFile(false);
		else
		{
			//writeToLog.addComment("index.dat could not be created! make sure that the dem.dat is the input directory");
			return false;

		}
	}

	std::ifstream readIndex(directory + "expdecayTI.dat");

	if (!readIndex)
	{
		//writeToLog.addComment("index.dat could not be opened!");
		return false;
	}

	double tempArea = -1111.0;
	double tempIndex = -1111.0;
	std::string space;
	getline(readIndex, space);
	readIndex >> indexSize >> subFarea;

	while (readIndex >> tempArea && readIndex >> tempIndex)
	{
		index.push_back(tempIndex);
		indexArea.push_back(tempArea);
	}

	readIndex.close();

	if (!aveIndex())
		return false;
	else
		return true;
}


bool TopographyData::calcIndexFile(std::string &directory, double N, std::string simulationMode, std::string routingAlgorithm)
{

	if (!fileExists(directory + "powdecayTI.dat"))
	{
		IndexCalculator calc(directory, N, simulationMode, routingAlgorithm);
		if (calc.getParseSucess())
			calc.createIndexFile(true);
		else
		{
			//writeToLog.addComment("index.dat could not be created! make sure that the dem.dat is the input directory");
			return false;
		}
	}
	std::ifstream readIndex(directory + "powdecayTI.dat");
	if (!readIndex)
	{
		//writeToLog.addComment("index.dat could not be opened!");
		return false;
	}

	double tempArea = -1111.0;
	double tempIndex = -1111.0;
	std::string space;
	getline(readIndex, space);
	readIndex >> indexSize >> subFarea;
	while (readIndex >> tempArea && readIndex >> tempIndex)
	{
		index.push_back(tempIndex);
		indexArea.push_back(tempArea);
	}
	readIndex.close();
	if (!aveIndex())
		return false;
	else
		return true;
}

bool TopographyData::aveIndex()
{
	averageIndex = 0.0;
	double totalArea = 0.0;
	for (size_t i = 0; i< index.size(); i++)
	{
		totalArea = totalArea + indexArea[i]; // total area of the subcatchment
	}

	if (totalArea <= 0.0)
	{
		//writeToLog.addComment("Error calculating the average index. Total area of subcatchment less than or equal to zero.");
		return false;
	}

	double SUMAC = 0.0;

	if (totalArea != 0.0)
		indexArea[0] = indexArea[0] / totalArea;   // fractional area associated with each ln(a/tanB)

	for (size_t i = 1; i < index.size(); i++)
	{
		indexArea[i] = indexArea[i] / totalArea;
		//        SUMAC = SUMAC + indexArea[i];
		averageIndex = averageIndex + indexArea[i] * (index[i] + index[i - 1]) / 2;
	}

	return true;
}




void TopographyData::calcPropVector()
{
	double flowDir;
	int location1, location2;

	for (int folder = 0; folder < FDorganizer_1D.size(); folder++)
	{
		flowDir = FDorganizer_1D[folder];
		if (flowDir != NODATA)
		{

			// I belive this works for transforming linearized organizer into [Row,Col]
			int col = folder % ncols;
			int row = folder / ncols;

			double proportion1 = 0;
			double proportion2 = 0;
			int a1 = 0;
			int b1 = 0;
			int a2 = 0;
			int b2 = 0;


			if (flowDir >= 0) {
				// find which two folders receive flow and the proportion to each
				if (flowDir >= 0 && flowDir < 45) {
					proportion1 = (45 - flowDir) / 45;
					a1 = col;
					b1 = row - 1;
					proportion2 = flowDir / 45;
					a2 = col + 1;
					b2 = row - 1;
				}
				else if (flowDir >= 45 && flowDir < 90) {
					proportion1 = (90 - flowDir) / 45;
					a1 = col + 1;
					b1 = row - 1;
					proportion2 = (flowDir - 45) / 45;
					a2 = col + 1;
					b2 = row;
				}
				else if (flowDir >= 90 && flowDir < 135) {
					proportion1 = (135 - flowDir) / 45;
					a1 = col + 1;
					b1 = row;
					proportion2 = (flowDir - 90) / 45;
					a2 = col + 1;
					b2 = row + 1;
				}
				else if (flowDir >= 135 && flowDir < 180) {
					proportion1 = (180 - flowDir) / 45;
					a1 = col + 1;
					b1 = row + 1;
					proportion2 = (flowDir - 135) / 45;
					a2 = col;
					b2 = row + 1;
				}
				else if (flowDir >= 180 && flowDir < 225) {
					proportion1 = (225 - flowDir) / 45;
					a1 = col;
					b1 = row + 1;
					proportion2 = (flowDir - 180) / 45;
					a2 = col - 1;
					b2 = row + 1;
				}
				else if (flowDir >= 225 && flowDir < 270) {
					proportion1 = (270 - flowDir) / 45;
					a1 = col - 1;
					b1 = row + 1;
					proportion2 = (flowDir - 225) / 45;
					a2 = col - 1;
					b2 = row;
				}
				else if (flowDir >= 270 && flowDir < 315) {
					proportion1 = (315 - flowDir) / 45;
					a1 = col - 1;
					b1 = row;
					proportion2 = (flowDir - 270) / 45;
					a2 = col - 1;
					b2 = row - 1;
				}
				else if (flowDir >= 315 && flowDir <= 360) {
					proportion1 = (360 - flowDir) / 45;
					a1 = col - 1;
					b1 = row - 1;
					proportion2 = (flowDir - 315) / 45;
					a2 = col;
					b2 = row - 1;
				}


				// recording the proportion that flows to each downstream folder
				location1 = ncols * b1 + a1;
				location2 = ncols * b2 + a2;
				std::pair<int, int> downstreamfolders = std::make_pair(location1, location2);
				downstreamfoldermap.emplace(folder, downstreamfolders);
				std::pair<int, int> downstreamProportion = std::make_pair(proportion1, proportion2);
				downstreamfoldermap.emplace(folder, downstreamProportion);
			}

		}
		else {

			std::pair<int, int> downstreamfolders = std::make_pair(NODATA, NODATA);
			downstreamfoldermap.emplace(folder, downstreamfolders);
			std::pair<int, int> downstreamProportion = std::make_pair(NODATA, NODATA);
			downstreamfoldermap.emplace(folder, downstreamProportion);
		}
	}


}


