#ifndef INPUTTOP_H
#define INPUTTOP_H
//////////////////////////////////////////////////////////////////////////////////////
//																		            //
// InputTopographyData.h - Calculates and/or stores topography data                 //																		   
// ver 1.0                                                                          //
// Language:    Visual C++ 2010                                                     //
// Platform:    MacBookPro5,4  Win7										            //
// Application: HydroPrime/iHydro Version6, Summer 2012                             //
// Programmer:  				            //																	   	
//////////////////////////////////////////////////////////////////////////////////////
/*


Origin:
==================
Program: iHydro Version 5 & iHydro Version 4.5

Source Code File(s):  V4.5-index.h/.cpp indexN.h/.cpp
V5-index.h/.cpp  indexN.h/.cpp


Author(s): Jun Wang, jwang06@mailbox.syr.edu, Original author V4/V4.5
Yang Yang, yyang31@syr.edu, V5
 
*Incorporated calculations into new architecture
Creation Date: Summer 2000

Publication(s): Jun Wang PHDThesis



Module Operations:
==================



Public Interface:
=================


Build Process:
==============
Required files
- InputTopographyData.cpp, InputParameters.h, WriteLogFile.h
Build commands
- devenv hydroprime.sln

Maintenance History:
====================

*/

#include <string>
#include <utility>
#include <map>
#include <vector>
#include <sstream>
#include "IndexCalculator.h"


class TopographyData
{

public:

	TopographyData(std::string directory, std::string simulationMode, std::string routingAlgorithm, double numfolders, double simulationNoDataVal);

	void init(double N);
	double getAveSoilMoistureDeficit(double topAveIndex, double T0, double mM, double Q0, double nN);

	std::vector<double> elevation1D;
	std::vector<double> slope1D;
	std::vector<double> aspect1D;
	std::vector<double> indexArea;
	std::vector<double> index;
	double averageIndex;
	double indexSize;
	bool readSlopeAspectFile();
	int getcols () {return nDemDataCols;}
	int getrows () {return nDemDataRows;}
	double getNODATA () {return NODATA;}
	double getxllcorner(){return nDemDataxllcorner;}
	double getyllcorner(){return nDemDatayllcorner;}

	void setcols(int x) { nDemDataCols = x; }
	void setrows(int x) { nDemDataRows = x; }
	void setxllcorner(double x) { nDemDataxllcorner = x; }
	void setyllcorner(double x) { nDemDatayllcorner = x; }

	

private:

	double numfolders;
	double simulationNoDataVal;
	std::string simulationMode;
	std::string routingAlgorithm;
	int ncols;
	int nrows;
	double foldersize;
	double NODATA;
	double N;
	double subFarea;
	double folderSize;
	std::string directory;
	
	int nDemDataCols;
	int nDemDataRows;
	double nDemDataxllcorner;
	double nDemDatayllcorner;

	std::vector<double> FDorganizer_1D;

	std::map<int, std::pair<int, int>> downstreamfoldermap;
	std::map<int, std::pair<double, double>> downstreamProportionmap;

	bool fileExists(std::string path);
	bool calcIndexFile(std::string &directory, double N, std::string simulationMode, std::string routingAlgorithm);
	bool calcIndexFile(std::string &directory, std::string simulationMode, std::string routingAlgorithm);
	bool calcDistIndexFile(std::string &directory, double N, std::string simulationMode, std::string routingAlgorithm);
	bool calcDistIndexFile(std::string &directory, std::string simulationMode, std::string routingAlgorithm);
	bool aveIndex();
	bool distAveIndex();
	void calcPropVector();

};

#endif