#ifndef TREEINTERCEPT_H
#define TREEINTERCEPT_H
///////////////////////////////////////////////////////////////////////////////////////
//																		             //
// TreeInterception.h - Calculates tree interception                                 //																		   
// ver 1.0                                                                           //
// Language:    Visual C++ 2010                                                      //
// Platform:    MacBookPro5,4  Win7										             //
// Application: HydroPrime/iHydro Version6, Summer 2012                              //
// Programmer:  				             //																	   	
///////////////////////////////////////////////////////////////////////////////////////
/*
  Origin:
  ==================
  Program: iHydro Version 5
  
  Source Code File(s): VegInterception.cpp
  Functions:           V5 - (line 81) Treeintercept()
  
  Author(s): Yang Yang, yyang31@syr.edu, V5
			  	
						*Incorporated calculations into new architecture
  Creation Date: 03 June 2010

  Publication(s): 

  Module Operations: 
  ==================
  

  Public Interface:
  =================
  TreeInterceptionCalc(Inputs *input, Repo *repo);
  void calculate(Inputs *input, DataFolder *folder, int timeStep);

  Build Process:
  ==============
  Required files
	-ICalc.h, InputsRepository.h, ResultsRepository.h, TreeInterception.cpp 
				
  Build commands 
    - devenv hydroprime.sln
   
  Maintenance History:
  ====================
  */

#include "..\Inputs\Inputs.h"
#include "..\DataFolder.h"

class TreeInterceptionCalc 
{
	// TreeInterceptionCalc is a derived class from the base class ICalc
	// TreeInterceptionCalc inherits all member data and member functions from base class ICalc
	// The only function to inherit, and required to be included, is the calculate function
	public:
		static void calculate(Inputs *input, DataFolder *folder, int timeStep);


	private:
	    static void treeInterceptionRain(Inputs *input, DataFolder *folder, int timeStep);				// Member function that determines intercepted rain
		static void treeInterceptionSnow(Inputs *input, DataFolder *folder, int timeStep);				// Member function that determines intercepted snow
		static void treeUnloadSnow(Inputs *input, DataFolder *folder, int timeS);						// Member function that determines unloaded snow
		static void treeSnowMeltbyRn(Inputs *input, DataFolder *folder, int timeStep);					// Member function that determines snow melt by rain
		static void storeValues(Inputs *input, DataFolder *folder, int timeStep);
};
#endif