#include "RootZoneFlowCalc.h"


void  RootZoneFlowCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	//  The amount of water infiltrated from the saturation excess routine (basically all overflow from pervious depression storage) * % of watershed governed by saturation excess
	double satExcessInflow =  folder->VarDict["runoffToInfil_postPMacro"] * (1 - folder->ParamDict["InfiltExcessGovernedArea_frac"]);
	// The amount of water infiltrated from the infiltration excess routine * % of watershed governed by infiltration excess
	double infilExcessInfLow = folder->VarDict["infilExInfil"] * folder->ParamDict["InfiltExcessGovernedArea_frac"];
	// Total flow intoRootZone
	double flowIntoRootZone = satExcessInflow + infilExcessInfLow;
	folder->VarDict["flowIntoRootZone"] = flowIntoRootZone;
	if (folder->ParamStringDict["Type"] == "PermeablePavement") {
		folder->VarDict["flowIntoRootZone"] = folder->VarDict["pavePerc"];
	}
	
	// Macropore Warning a mismatch bewteen the ncols in the param.dat and the TreeCover.txt has occure
	double pMacroFlow = folder->VarDict["pMacroFlow"];

	// storing the infil values
	if (input->SimulationStringParams["ExtendedOutputsTimeStep"] != "NoExtendedOutputs")
	{
		folder->VarVecDict["satExInfil"].emplace_back(satExcessInflow);
		folder->VarVecDict["infilExInfil"].emplace_back(infilExcessInfLow);
		folder->VarVecDict["infilWeighted"].emplace_back(flowIntoRootZone);
	}

	/*
	// Output to track the flow heaing into the rootzone
	std::ofstream outfileInflow("InfiltrationToRootzone.dat", std::ios::app);
	if(!outfileInflow.good())
		return;
	outfileInflow	<<"  "<<std::std::setw(13) <<"SatExInflow"
		<<"  "<<std::std::setw(13) <<satExcessInflow
		<<"  "<<std::std::setw(13) <<"InfilExInflow"
	    <<"  "<<std::std::setw(13) <<infilExcessInfLow<<std::endl;

	*/

	// This for loop runs for each member of the Topographic Index Distribution - typically 30 values
	for(int ia=0; ia< input->IndexAreaSize; ia++)
	{
		if (timeStep == 0)
		{
			double initRootZoneDef = ((folder->ParamDict["SD0_percent"] / 100)  *  folder->ParamDict["MSD_m"]);
			folder->VarVecDict["rootZoneQ_TI"].push_back(0.0);
			folder->VarVecDict["rootZoneStorDef_TI"].push_back(initRootZoneDef);
		}
		folder->VarVecDict["rootZoneQ_TI"][ia] = pMacroFlow;   // Setting the flow flow/through the root zone for this index value to 0
		// Modifying the rootzone deficit (a non-zero positive term indicates non-saturation)
		// Update the rootzone deficit ( rootzone deficit(for a folder, with the given index value) - R, for that folder, for that index value)
		// Q: Need to consider if this works for the distributed version, as each folder will only have 1 ia value - not the whole distribution
		folder->VarVecDict["rootZoneStorDef_TI"][ia] = folder->VarVecDict["rootZoneStorDef_TI"][ia] - flowIntoRootZone; // - septQ;   <- Q: is this part of the septic Organizer calc? Tom

		// if the updated rootzone deficit is negative (it is saturated)
		if(folder->VarVecDict["rootZoneStorDef_TI"][ia] < 0.0 )         
		{
			// modify flow out of the root zone to equal the the amount past saturation
			// i.e. if RzsDeficit is -0.3, then modify rzQ to be = 0 (previously set value) - -0.3 (RzsDeficit), gives and rzQ value of 0.3
			// this means that 0.3 m (or mm?) of flow is leaving the root zone due to its oversaturation
			folder->VarVecDict["rootZoneQ_TI"][ia] = (folder->VarVecDict["rootZoneQ_TI"][ia] - folder->VarVecDict["rootZoneStorDef_TI"][ia]);    // flow from root zone
			// Update rootzone storage deficit to be 0 - it is saturated, there is no deficit
			folder->VarVecDict["rootZoneStorDef_TI"][ia] = 0.0;
		}
	}

	
	storeValues(input, folder, timeStep);

}


void  RootZoneFlowCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	// Storing the rootzone flow (into unsaturated zone) at this timestep for this folder
	// Using 0 because the root zone flow is the same for 

	//folder->VarDict["totalRootZoneQ"] += folder->VarDict["rootZoneQ"]; 
	folder->VarDict["prevRootZoneQ"] = folder->VarVecDict["rootZoneQ_TI"][0];
	//folder->VarVecDict["rootZoneQ"].emplace_back(folder->VarVecDict["rootZoneQ_TI"][0]);
		
}

