#include "SemiAveSMDCalc.h"

void SemiAveSMDCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	double avgIndex = input->AverageIndex;

	// Reza: I had to add this if statement to solve the issue of T0 in AvgSMD in input.cpp
	if (timeStep == 0) {
		double T0 = folder->ParamDict["T0_m2ph"];
		double mM = folder->ParamDict["m"];
		double nN = folder->ParamDict["n"];
		double Q0 = folder->ParamDict["Q0_mph"];

		
		// double topAveIndex, double T0, double mM, double Q0, double nN) :
		double aveSMD = 0.0;
		if (input->SimulationStringParams["Infiltration"] == "PowerDecay")
		{
			// used for power decay calcualtions
			double SZQ = 0.0;
			//Q0 = 1*pow(10, -5);    // For testing purposes 
			SZQ = pow(Q0 / T0, 1 / nN)*(avgIndex);

			if (SZQ > 1.0)
				SZQ = 1.0;
			if (SZQ < 0.0)
				SZQ = 0.0;
			aveSMD = mM * (1.0 - SZQ);
		}
		else
		{
			// used for exponential decay calculations
			double SZQ = 0.0;
			SZQ = T0 * exp(-avgIndex);

			aveSMD = -mM * log(Q0 / SZQ);
		}
		folder->VarDict["aveSMD"] = aveSMD;
	}

	//folder->VarVecDict["aveSMD"].push_back(folder->VarDict["aveSMD"]);
	double t1 = folder->VarDict["unsatZoneQ"];
	double t2 = folder->VarDict["baseFlow"];
	folder->VarDict["aveSMD"] = folder->VarDict["aveSMD"] - folder->VarDict["unsatZoneQ"] + folder->VarDict["baseFlow"];
	
	// Reza: For the GIs we need to calculate the area percentage to adjust the AveSMD of the BA
	if (folder->ParamStringDict["Type"] != "BulkArea") {
		input->RepoDict["GI_Area"] = folder->ParamDict["Area"] * folder->ParamDict["giNumber"];
		input->RepoDict["GI_Area_percentage"] = input->RepoDict["GI_Area"] / input->SimulationNumericalParams["CatchmentArea_m2"];
		// This way the model will gather all the localSMD_TI_GI from the all GI devices.
		if (timeStep == 0){ 
			folder->VarDict["AveSMD_GI"] = folder->VarDict["aveSMD"];
			if (folder->VarDict["AveSMD_GI"] < 0.0) {
				folder->VarDict["AveSMD_GI"] = 0.00001;
			}
			input->RepoDict["AveSMD_GI_Total"] += folder->VarDict["AveSMD_GI"];
		}
		input->RepoDict["AveSMD_GI_Total"] += folder->VarDict["AveSMD_GI"];
		//folder->VarDict["aveSMD"] = input->RepoDict["AveSMD_GI"];
	}
	if (input->RepoDict["DataFolderNm"] != 1.0 && 
		folder->ParamStringDict["Type"] == "BulkArea") {
		
		folder->VarDict["aveSMD"] = (((100.0 - input->RepoDict["GI_Area_percentage"]) / 100.0) * folder->VarDict["aveSMD"]) +
			(((input->RepoDict["GI_Area_percentage"]) / 100.0) * input->RepoDict["AveSMD_GI_Total"]);

	}

	
	if (folder->VarDict["aveSMD"] < 0.0)
	{
		folder->VarDict["aveSMD"] = 0.00001;
	}



}
