#ifndef RESISTANCECAL_H
#define RESISTANCECAL_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
//  ResistanceCal.h --  calculate heat flux resistances                 //
//                                                                      //
//                                                                      //
//  Language:     Visual Studio 2010 C++                                //
//                                                                      //
//  Application:  iTree-Temperature model                               //
//  Author:       Yang Yang                                             //
//                yyang31@syr.edu                                       //
//                                                                      //
//////////////////////////////////////////////////////////////////////////


#include <iostream>
#include <vector>
#include <string>
#include <math.h>
#include "..\Inputs\Inputs.h"

using namespace std;                 

class ResistanceCal
{
	private:
	   
		double rtree;               // resistance from tree canopy to folder canopy layer
		double rShortVeg;               // resistance from short vegetation canopy to folder canopy layer
		double rimp;
		double rground;             //resistance from ground to folder canopy layer
		double rcanopy;            // aerodynamic resistance from the whole folder canopy to mesoscale air layer
       // resistances that depend on the water content in the soil, so they are different for each folder
		double rg;                 // soil resistance
		double rctree;           // tree canopy resistance
		double rcshort;           // short vegetation canopy resistance


		//parameters used to calculate the resistances

	double satPoint;
	double wiltPoint;
	double fieldPoint;
	double MSD;
	double TreeMinR;
	double ShortMinR;
	double Ri;    //Richardson number
	double F;    // atmopshere stability correction number 
	double Zref;
	double Treew; // average tree leaf width
	double Shortw;  // average short vegetation leaf width

	//constants
	double lamda;
	double Cp;
	double rou;
	double k;
	double g; 
	double ak;
	double a;
	double rD;     // roughness length of ground
	//

public:
		
	ResistanceCal(Inputs* input);
		~ ResistanceCal(){};
		void cal_factors(double Tb, double Tair, double Uref, double aveHeight,double height);
		void calr_rural(double Uref, double treeLAI, double shortLAI,double aveHeight);    // calculate the resistances that do not depend on soil water content
		void calr_urban(double Uref, double treeLAI, double shortLAI, double aveHeight, double hwR,double height);
		void calr_onwater(double SoilWaterContent, double treeLAI, double shortLAI);  //calculate the resistance that depend on the soil water content
		double get_F(){return F;}
		friend class HeatFluxCal;
};



#endif