//////////////////////////////////////////////////////////////////////////	
//  TemperatureOutputWriter.cpp - Provides methods to write Temperature	//
//					             related output files				    //
//////////////////////////////////////////////////////////////////////////

#include "TemperatureOutputWriter.h"
#include <fstream>
#include <iomanip>




void TemperatureOutputWriter::timeStepResultsFileProcessor(int timeStep, DataOrganizer* organizer, std::string outDirectory, Inputs* input)
{

	bool print = input->TemperatureCalculationParams["TimeBasedOutputAllTimeSteps"];
	int hr = timeStep % 24;
	int currDay = input->CalendarDates[timeStep] * 100 + hr;
	if (!print)
	{
		for (int iter = 0; iter < input->StartDatesForTimeBasedTempOutput.size(); ++iter)
		{
			if (currDay >= input->StartDatesForTimeBasedTempOutput[iter] && currDay <= input->EndDatesForTimeBasedTempOutput[iter])
			{
				print = true;
				break;
			}
		}
	}
	if (print)
		createTimeStepResultFile(input->TemperatureExecutionParams["PrintTimeBasedResults"], currDay, input);
	if (print && input->TemperatureCalculationParams["PrintAllTimeBasedResults"] == 1)
	{
		createTimeStepResultFile("AirT", currDay, input);
		createTimeStepResultFile("Td", currDay, input);
		createTimeStepResultFile("AirE", currDay, input);
		createTimeStepResultFile("ImpNR", currDay, input);
		createTimeStepResultFile("TreeNR", currDay, input);
		createTimeStepResultFile("ShortVegNR", currDay, input);
		createTimeStepResultFile("SoilNR", currDay, input);
		createTimeStepResultFile("WaterNR", currDay, input);
		createTimeStepResultFile("ImperviousH", currDay, input);
		createTimeStepResultFile("ImperviousLE", currDay, input);
		createTimeStepResultFile("TreeH", currDay, input);
		createTimeStepResultFile("TreeLE", currDay, input);
		createTimeStepResultFile("TreeLEE", currDay, input);
		createTimeStepResultFile("TreeLET", currDay, input);
		createTimeStepResultFile("ShortVegH", currDay, input);
		createTimeStepResultFile("ShortVegLE", currDay, input);
		createTimeStepResultFile("ShortVegLEE", currDay, input);
		createTimeStepResultFile("ShortVegLET", currDay, input);
		createTimeStepResultFile("SoilH", currDay, input);
		createTimeStepResultFile("SoilLE", currDay, input);
		createTimeStepResultFile("WaterH", currDay, input);
		createTimeStepResultFile("WaterLE", currDay, input);
		createTimeStepResultFile("H", currDay, input);
		createTimeStepResultFile("LE", currDay, input);

		createTimeStepResultFile("ImpDeltaQ", currDay, input);
		createTimeStepResultFile("TreeDeltaQ", currDay, input);
		createTimeStepResultFile("ShortDeltaQ", currDay, input);
		createTimeStepResultFile("SoilDeltaQ", currDay, input);
		//createTimeStepResultFile("WaterDeltaQ", currDay, input);

	}
	if (print)
	{
		writeToTimeStepResultFile(input->TemperatureExecutionParams["PrintTimeBasedResults"], currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
	}
	if (print && input->TemperatureCalculationParams["PrintAllTimeBasedResults"])
	{
		writeToTimeStepResultFile("AirT", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("Td", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("AirE", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ImpNR", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeNR", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegNR", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("SoilNR", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("WaterNR", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ImperviousH", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ImperviousLE", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeH", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeLE", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeLEE", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeLET", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegH", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegLE", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegLEE", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegLET", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("SoilH", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("SoilLE", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("WaterH", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("WaterLE", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("H", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("LE", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ImpDeltaQ", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeDeltaQ", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortDeltaQ", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("SoilDeltaQ", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("WaterDeltaQ", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
	}
}

void TemperatureOutputWriter::rowColFileBatchProcessor(int timeStep, int organizerLocationIndex, DataFolder* folder, std::string outDirectory, Inputs* input)
{

	bool locationDataIsToBeOutputted = input->TemperatureCalculationParams["PrintAllRowColOutput"];
	if (!locationDataIsToBeOutputted)
		locationDataIsToBeOutputted = input->LocationIdsToPrintTempOutput.find(organizerLocationIndex) != input->LocationIdsToPrintTempOutput.end();
	if (!locationDataIsToBeOutputted)
		return;
	int row = (int)(organizerLocationIndex / (int)(input->LocationParams["nCols"]) + 1);
	int col = (organizerLocationIndex % (int)(input->LocationParams["nCols"]) + 1);
	bool printLocation = input->TemperatureCalculationParams["RowColOutputAllTimeSteps"];

	//control to write out 1st time step, connects with TestSimulation.cpp statement: if (timeStep > 0)
	if (timeStep > 0) {
		int hr = timeStep % 24;
		int currDay = input->CalendarDates[timeStep] * 100 + hr;

		if (!printLocation)
		{
			for (int iter = 0; iter < input->StartDatesForRowColTempOutput.size(); ++iter)
			{
				if (currDay >= input->StartDatesForRowColTempOutput[iter] && currDay <= input->EndDatesForRowColTempOutput[iter])
				{
					printLocation = true;
					break;
				}
			}
		}
		if (printLocation)
		{
			writeToHeatRowColFile(currDay, row, col, folder, input->SimulationStringParams["OutputDirectory"]);
			writeToHydroRowColFile(currDay, row, col, folder, input->SimulationStringParams["OutputDirectory"]);
		}
	}
}




void TemperatureOutputWriter::createHydroRowColFile(int i, int j, std::string outDirectory, Inputs* input)
{
	i -= 1;
	j -= 1;
	char str[40];
	strcpy(str, "\\Row");
	char str1[10];
	itoa(i, str1, 10);
	strcat(str, str1);
	strcat(str, "Col");
	char str2[10];
	itoa(j, str2, 10);
	strcat(str, str2);
	strcat(str, "Hydro.txt");
	ofstream outfile(outDirectory + str);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot write to " + outDirectory + str);
		return;
	}
	//  std::cout<<"Writing outputs.txt file\n"<<std::endl;
	int col = input->LocationParams["nCols"];
	outfile << "Elevation:  " << input->DEM[i * col + j] << endl;
	outfile << "Slope:  " << input->Slope[i * col + j] << endl;
	outfile << "Aspect:  " << input->Aspect[i * col + j] << endl;
	outfile << "AverageIndex: " << input->AverageIndex << endl;
	outfile << "TI:  " << input->Index[i * col + j] << endl;
	outfile << "Land cover type:  " << input->NLCDLandCoverData[i * col + j] << endl;
	outfile << "Tree cover percentage:  " << input->TreeCoverdata[i * col + j] << endl;
	//outfile <<"Short vegetation cover percentage:  "<<repo->input->getSpData()->getImpCover(i*col+j)<<endl;
	outfile << "Impervious cover percentage:  " << input->ImperCoverdata[i * col + j] << endl;
	outfile << "WOnImp    " << "WOnTree    " << "WOnShort    " <<
		"Cumfil    " << "FilExQ    " << "RootStoreD    " << "OverLandQ    " <<
		"LocalSMD    " << "uszStore    " <<
		"Tp    " << "Ponding    " << "ImpDrunoff    " << endl;
	//      int totalts = input->SimulationNumericalParams["TotalTimeSteps"];
	outfile.close();
	LogMessage::log("Created " + outDirectory + str);
}


void TemperatureOutputWriter::writeToHydroRowColFile(int timestemp, int i, int j, DataFolder* folder, std::string outDirectory) {
	i -= 1;
	j -= 1;
	char str[40];
	strcpy(str, "\\Row");
	char str1[10];
	itoa(i, str1, 10);

	//cout << "value of i and j is" << i << "  " << j << endl;
	//cout << "wate on impervious value is" << folder->VarVecDict["rootZoneStorDef_TI"].at(0) << endl;
	//cout << "wate on impervious value is" << folder->VarVecDict["adjPerAreaFlow"].at(0) << endl;
	//cout << "wate on impervious value is" << folder->VarVecDict["localSMD"].at(0) << endl;
	//cout << "wate on impervious value is" << folder->VarVecDict["rootZoneStorDef_TI"].at(0) << endl;

	strcat(str, str1);
	strcat(str, "Col");
	char str2[10];
	itoa(j, str2, 10);
	strcat(str, str2);
	strcat(str, "Hydro.txt");
	ofstream outfile(outDirectory + str, ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot write to " + outDirectory + str);
		return;
	}
	//outfile << folder->VarDict["WaterOnImpervious"] << "  "
	//  << folder->VarDict["WaterOnTree"] << "  " <<
	//  folder->VarDict["WaterOnShortVeg"] << "  " <<
	//  folder->VarDict["Cumfil"] << "  " <<
	//  folder->VarDict["FilExcessQ"] << "  " <<
	//  folder->VarDict["RootZoneStorageD"] << "  " <<
	//  folder->VarDict["OverlandQ"] << "  " <<
	//  folder->VarDict["localSMD"] << "  " <<
	//  folder->VarDict["uszStorage"] << "  " <<
	//  folder->VarDict["TP"] << "  " <<
	//  folder->VarDict["ponding"] << "  " <<
	//  folder->VarDict["ImperDirRunoff"] << endl;

//	outfile << folder->VarDict["waterOnImperviousArea"] << " ";
	outfile << folder->VarDict["impDepStor_m"] << " ";
	outfile << folder->VarDict["treeStoredW"] << "  ";
	outfile << folder->VarDict["shortVegStoredW"] << "  ";
	outfile << folder->VarDict["cumInfil"] << "  ";
	outfile << folder->VarDict["adjInfilExRunoff"] << "  ";
	outfile << folder->VarVecDict["rootZoneStorDef_TI"].at(0) << "  ";
	outfile << folder->VarVecDict["satExRunoff_TI"].at(0) << "  ";
	outfile << folder->VarVecDict["localSMD_TI"].at(0) << "  ";
	outfile << folder->VarVecDict["unsatZoneStor_TI"].at(0) << "  ";
	outfile << folder->VarDict["pondTime"] << "  ";
	outfile << folder->VarDict["ponding"] << "  ";
	outfile << folder->VarDict["ImperDirRunoff"] << endl;
	outfile.close();
}


void TemperatureOutputWriter::createBlockGroupHourlyFile(std::string outDirectory, int blockGroupIndex)
{
	std::string str = "BlockGroup" + std::to_string(blockGroupIndex) + ".txt";
	ofstream outfile(outDirectory + "\\" + str);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Could not create BlockGroup" + std::to_string(blockGroupIndex) + ".txt");
		return;
	}

	outfile << "YYYYMMDDHH    " << "Ta    " << "Td" << endl;

	outfile.close();

}

void TemperatureOutputWriter::writeToBlockGroupHourlyFile(std::string outDirectory, std::string date, DataFolder* blockGroupFolder)
{
	int bgId = static_cast<int>(blockGroupFolder->ParamDict["blockGroupIndex"]);
	std::string str = "BlockGroup" + std::to_string(bgId) + ".txt";

	ofstream outfile(outDirectory + "//" + str, ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write " + str + " file" << std::endl;
		LogMessage::log("Could not write to " + str);
		return;
	}

	outfile << date << "   " << blockGroupFolder->VarDict["AirT"] / blockGroupFolder->ParamDict["folderCount"] << "  "
		<< blockGroupFolder->VarDict["Td"] / blockGroupFolder->ParamDict["folderCount"] << endl;

}

void TemperatureOutputWriter::writeToBlockGroupDailyFile(std::string outDirectory, std::string date, DataFolder* blockGroupFolder, int timeStep)
{
	int bgId = static_cast<int>(blockGroupFolder->ParamDict["blockGroupIndex"]);
	std::string str = "BlockGroupDaily" + std::to_string(bgId) + ".txt";
	ofstream outfile(outDirectory + "//" + str, ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write " + str + " file" << std::endl;
		LogMessage::log("Cannot write " + str);
		return;
	}
	int avgDenom = (timeStep != 23) ? 24 : 23;

	//HeatIndex2 terms are from Lin, S., M. Luo, R. J. Walker, X. Liu, S. A. Hwang and R. Chinery (2009)
	//HeatIndex2bAvg calculated with daily average Tair and Tdew
	//HeatIndex2Avg is daily average heat index calculated as average of hourly heat index values, and is within 0.1% of HeatIndex2bAvg 
	double HeatIndex2bAvg = -2.653 + 0.994 * ((blockGroupFolder->VarDict["AirTAvg"] / avgDenom) - 273.15) + 0.0153 * pow(((blockGroupFolder->VarDict["TdAvg"] / avgDenom) - 273.15), 2);
	outfile << date << "   "
		<< blockGroupFolder->VarDict["AirTMax"] << "  "
		<< blockGroupFolder->VarDict["AirTMin"] << "  "
		<< blockGroupFolder->VarDict["AirTAvg"] / avgDenom << "  "
		<< blockGroupFolder->VarDict["TdAvg"] / avgDenom << "  "
		<< blockGroupFolder->VarDict["HeatIndexMax"] << "  "
		<< blockGroupFolder->VarDict["HeatIndexAvg"] / avgDenom << "  "
		<< blockGroupFolder->VarDict["HeatIndex2Max"] << "  "
		<< blockGroupFolder->VarDict["HeatIndex2Avg"] / avgDenom << "  "
		<< (HeatIndex2bAvg + 273.15) << "  "
		<< blockGroupFolder->VarDict["WindChillMin"] << "  "
		<< blockGroupFolder->VarDict["WindChillAvg"] / avgDenom << endl;

	outfile.close();

}

void TemperatureOutputWriter::createBlockGroupDailyFile(std::string outDirectory, int blockGroupIndex)
{
	std::string str = "BlockGroupDaily" + std::to_string(blockGroupIndex) + ".txt";
	ofstream outfile(outDirectory + "\\" + str);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot write to BlockGroupDaily" + std::to_string(blockGroupIndex) + ".txt");
		return;
	}

	//HeatIndex terms are from Rothfusz, Lans P. National Weather Service Technical Attachment SR 90-23 (1990) 
	//HeatIndex2 terms are from Lin, S., M. Luo, R. J. Walker, X. Liu, S. A. Hwang and R. Chinery (2009)
	//HeatIndex2Avg_Lin is daily average heat index calculated as average of hourly heat index values
	//HeatIndex2bAvg_Lin calculated with daily average Tair and Tdew
	outfile << "YYYYMMDD    " << "RawTairMax    "
		<< "RawTairMin    " << "RawTairAvg    "
		<< "RawTdewAvg    " << "HeatIndexMax_NWS "
		<< "HeatIndexAvg_NWS " << "HeatIndex2Max_Lin "
		<< "HtIdx2AvgDlyHI_Lin " << "HtIdx2DlyAvgTaTd_Lin " << "WindchillMin    "
		<< "WindchillAvg" << endl;

	outfile.close();

}

void TemperatureOutputWriter::createHeatRowColFile(int i, int j, std::string outDirectory) {
	i -= 1;
	j -= 1;
	char str[40];
	strcpy(str, "\\Row");
	char str1[10];
	itoa(i, str1, 10);
	strcat(str, str1);
	strcat(str, "Col");
	char str2[10];
	itoa(j, str2, 10);
	strcat(str, str2);
	strcat(str, "Heat.txt");
	ofstream outfile(outDirectory + str);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot create " + outDirectory + str);
		return;
	}
	//	std::cout<<"Writing outputs.txt file\n"<<std::endl;


	outfile << "ts    " << "Ta    " << "Td   " << "Ea    "
		//<<"rctree    "<<"rg    "<<"rcshrot    "
		<< "ImpNR    " << "TreeNR    " << "ShortVegNR    " <<
		"SoilNR    " << "WaterNR    " << "ImpH    " << "ImpLE    " <<
		"TreeH    " << "TreeLE    " << "TreeLEE    " <<
		"TreeLET    " << "ShortH    " << "ShortLE   " <<
		"ShortLEE    " << "ShortLET    " << "SoilH    " << "SoilLE  " <<
		"WaterH    " << "WaterLE  " << "H   " << "LE" << endl;
	outfile.close();
}

void TemperatureOutputWriter::generateRowColFiles(DataOrganizer* organizer, Inputs* input)
{
	for (int locationIndex = 0; locationIndex < organizer->DataDrawers.size(); ++locationIndex)
	{
		for (int folderIndex = 0; folderIndex < organizer->DataDrawers[locationIndex].size(); ++folderIndex)
		{
			int row = (int)(locationIndex / (int)(input->LocationParams["nCols"]) + 1);
			int col = (locationIndex % (int)(input->LocationParams["nCols"]) + 1);
			if (organizer->DataDrawers[locationIndex][folderIndex]->ParamStringDict["Type"] == "NODATA")
			{
				continue;
			}
			bool locationDataIsToBeOutputted = input->TemperatureCalculationParams["PrintAllRowColOutput"];
			if (!locationDataIsToBeOutputted)
				locationDataIsToBeOutputted = input->LocationIdsToPrintTempOutput.find(locationIndex) != input->LocationIdsToPrintTempOutput.end();
			if (locationDataIsToBeOutputted)
			{
				createHeatRowColFile(row, col, input->SimulationStringParams["OutputDirectory"] + "\\");
				createHydroRowColFile(row, col, input->SimulationStringParams["OutputDirectory"] + "\\", input);
			}
		}
	}
}




void TemperatureOutputWriter::writeToHeatRowColFile(int timeStep, int i, int j, DataFolder* folder, std::string outDirectory)
{
	i -= 1;
	j -= 1;
	char str[40];
	strcpy(str, "\\Row");
	char str1[10];
	itoa(i, str1, 10);
	strcat(str, str1);
	strcat(str, "Col");
	char str2[10];
	itoa(j, str2, 10);
	strcat(str, str2);
	strcat(str, "Heat.txt");
	ofstream outfile(outDirectory + str, ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot write to " + outDirectory + str);
		return;
	}


	outfile << timeStep << "   "
		<< folder->VarDict["AirT"] << "  "
		<< folder->VarDict["Td"] << "   "
		<< folder->VarDict["AirE"] << "   "
		<< folder->VarDict["ImpNR"] << "  "
		<< folder->VarDict["TreeNR"] << "  "
		<< folder->VarDict["ShortVegNR"] << "  "
		<< folder->VarDict["SoilNR"] << "  "
		<< folder->VarDict["WaterNR"] << "  "
		<< folder->VarDict["ImperviousH"] << "  "
		<< folder->VarDict["ImperviousLE"] << "  "
		<< folder->VarDict["TreeH"] << "  "
		<< folder->VarDict["TreeLE"] << "  "
		<< folder->VarDict["TreeLEE"] << "  "
		<< folder->VarDict["TreeLET"] << "  "
		<< folder->VarDict["ShortVegH"] << "  "
		<< folder->VarDict["ShortVegLE"] << "  "
		<< folder->VarDict["ShortVegLEE"] << "  "
		<< folder->VarDict["ShortVegLET"] << "  "
		<< folder->VarDict["SoilH"] << "  "
		<< folder->VarDict["SoilLE"] << "  "
		<< folder->VarDict["WaterH"] << "  "
		<< folder->VarDict["WaterLE"] << "  "
		<< folder->VarDict["H"] << "  "
		<< folder->VarDict["LE"] << endl;

	outfile.close();
}

void TemperatureOutputWriter::createTimeStepResultFile(std::string outputVarName, int ts, Inputs* input)
{


	std::string fName = input->SimulationStringParams["OutputDirectory"] + "\\" + outputVarName + "_" + std::to_string(ts) + ".txt";
	std::ofstream outfile(fName);
	if (!outfile.good())
	{
		std::cout << "Cannot write file: " + fName << std::endl;
		LogMessage::log("Cannot write file: " + fName);
		return;
	}

	outfile << "ncols" << std::setw(20) << input->LocationParams["nCols"] << endl;
	outfile << "nrows" << std::setw(20) << input->LocationParams["nRows"] << endl;
	outfile << "xllcorner" << std::setw(20) << input->LocationParams["xllcorner"] << endl;
	outfile << "yllcorner" << std::setw(20) << input->LocationParams["yllcorner"] << endl;
	outfile << "cellsize" << std::setw(16) << input->LocationParams["cellsize"] << endl;
	outfile << "NODATA_value" << std::setw(15) << input->LocationParams["NODATA"] << endl;
	outfile.close();
}

void TemperatureOutputWriter::writeToTimeStepResultFile(std::string outputVarName, int ts, std::string& outDirectory, DataOrganizer* organizer, Inputs* input)
{
	std::string fName = outDirectory + "\\" + outputVarName + "_" + std::to_string(ts) + ".txt";
	std::ofstream outfile(fName, std::ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write file: " + fName << std::endl;
		LogMessage::log("Cannot write file: " + fName);
		return;
	}
	for (int i = 0; i < organizer->DataDrawers.size(); ++i)
	{
		DataFolder* folder = organizer->DataDrawers[i][0];
		int row = (int)(i / (int)(input->LocationParams["nCols"]) + 1);
		int col = (i % (int)(input->LocationParams["nCols"]) + 1);
		bool newLine = (col != (int)(input->LocationParams["nCols"])) ? false : true;
		if (folder->ParamStringDict["Type"] == "NODATA")
		{
			outfile << "-9999";
		}
		else
			outfile << folder->VarDict[outputVarName];
		if (newLine)
		{
			outfile << std::endl;
		}
		else
			outfile << " ";
	}
	outfile.close();
}

void TemperatureOutputWriter::generateBlockFiles(std::map<int, DataFolder*> blockGroupIDToItsLocation, std::string outDirectory, bool needHourlyBGFiles, bool createDailyFiles)
{
	for (std::map<int, DataFolder*>::iterator iter = blockGroupIDToItsLocation.begin(); iter != blockGroupIDToItsLocation.end(); ++iter)
	{
		if (needHourlyBGFiles)
		{
			createBlockGroupHourlyFile(outDirectory, iter->first);
		}
		if (createDailyFiles)
		{
			createBlockGroupDailyFile(outDirectory, iter->first);
		}
	}
}


void TemperatureOutputWriter::outputLandCoverDetails(int index, std::string outDirectory, DataFolder* folder)
{

	string str = outDirectory + std::to_string(index) + "LCScalingDetails.txt";
	std::fstream fs;
	if (folder->ParamDict["TreeP"] == 0 && folder->ParamDict["ImperP"] == 0)
		fs.open(str, std::ios_base::in | std::ios_base::out | std::ios_base::trunc);
	else
		fs.open(str, ios::app);
	fs << "TrP: " << folder->ParamDict["TreeP"]
		<< ",ImpP: " << folder->ParamDict["ImperP"]
		<< ",SoilP: " << folder->ParamDict["SoilP"]
		<< ",ShVgP: " << folder->ParamDict["ShortVegP"]
		<< ",WaterP: " << folder->ParamDict["WaterP"]
		<< ",ImpTr: " << folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"]
		<< ",PeTr: " << folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
		//<< ",ShVgFr:" << folder->ParamDict["ShortVegetationCover_frac"]
		//<< ",SoilFr:" << folder->ParamDict["SoilCover_frac"]
		//<< ",DCIA:" << folder->ParamDict["DCIA_frac"]
		<< std::endl;
	fs.close();
}
