#include "UnsatZoneFlowCalc.h"


void UnsatZoneFlowCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	// creating a temp variable for unsaturated zone flow
	double uzflow ;
	
	// intialize the current timestpes total - (semi-distirbuted) - to zero 
	folder->VarDict["unsatZoneQ"] = 0.0;

	// This for loop runs for each member of the Topographic Index Distribution - typically 30 values
	for(int ia=0; ia<input->IndexAreaSize; ia++)
	{   
		if (timeStep == 0)
		{
			folder->VarVecDict["unsatZoneQ_TI"].push_back(0.0);
		}
		// intializing the unsaturated zone flow to 0
		folder->VarVecDict["unsatZoneQ_TI"][ia] = 0.0;
		// setting the temp unsaturated zone flow to 0.0
		uzflow = 0.0;
	
		// if the local soil mositure deficit is > 0
		if (folder->VarVecDict["localSMD_TI"][ia] > 0.0 )
		{
			// unsaturated zone flow is (unsaturated zone storage / (local soil moisture deficit * Unsaturated zone time delay (Td) constant)
			// Calculation is from Beven and Wood (1983)
			uzflow = folder->VarVecDict["unsatZoneStor_TI"][ia] / (folder->VarVecDict["localSMD_TI"][ia] * folder->ParamDict["Td_h"]); 

			// if (unsaturated zone flow > unsaturated zone storage), set it equal to the unsaturated zone storage
			if ( uzflow > folder->VarVecDict["unsatZoneStor_TI"][ia])
				uzflow = folder->VarVecDict["unsatZoneStor_TI"][ia];                   // unsaturated zone storage <= 
			
			// modify the unsaturated zone storage to reflect the flow out of the unsaturated zone
			folder->VarVecDict["unsatZoneStor_TI"][ia] = folder->VarVecDict["unsatZoneStor_TI"][ia] - uzflow;
			// set the unsaturated zone flow for this TI bin
			folder->VarVecDict["unsatZoneQ_TI"][ia] = uzflow * input->IndexArea[ia];	
			
		}
		// Modify the total unsaturated zone flow to (previous total unsaturated zone flow + current unsaturated zone flow) of the whole watershed this timestep
		// Only useful in the Semi-Ditributed version - keeps track of the total for this timestep across all index bins
		// In the Distributed version is should always be the the same as the UszQ for a folder for that timestep
		folder->VarDict["unsatZoneQ"] += folder->VarVecDict["unsatZoneQ_TI"][ia];

		// Reza, for the output writer
		if (folder->ParamStringDict["type"] == "BulkArea") {
			input->RepoDict["adj_unsatZoneQ"] = folder->VarDict["unsatZoneQ"];
		}
	}

	// What's the purpose of this?
	// Updates the unsaturated zone flow count - increments it by 1
	// Not used right now - replaced by DistSatZoneUpdateProcess
	//repo->unsaturatedZone.setUszQcnt(repo->unsaturatedZone.getUszQcnt() + 1);


	
	// Added to the disributed version for tracking distributed unsat zone storage for each call across all timesteps
	//repo->unsaturatedZone.modifyUszStorage_Dist(stafolders, timeStep, repo->unsaturatedZone.getUszStorage(stafolders, 0));
	


}