#include "WaterOnPerviousAreaCalc.h"
#include <iostream>


void WaterOnPerviousAreaCalc::calculate(Inputs *input, DataFolder *folder, int timeStep, std::vector<DataFolder*> &stafolders)
{
	// TotalPerviousCover_frac = fraction of watershed with tree cover on pervious area + fraction of watershed with shortveg cover + bare soil area
	double TotalPerviousCover_frac;
	double TotalImperviousCover_frac;
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		TotalPerviousCover_frac = folder->ParamDict["TotalPerviousCover_frac"];
		TotalImperviousCover_frac = folder->ParamDict["TotalImperviousCover_frac"];
	}
	else {
		TotalPerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + 
			folder->ParamDict["ShortVegetationCover_frac"] +
			folder->ParamDict["SoilCover_frac"];
		TotalImperviousCover_frac = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCover_frac"];
	}

	// Computing weighted average of through flow ppt on pervious area results in actual depth of ppt, not fractional depth of ppt
	// 1st: summing 3 fractional area contributions gets fractional depth
	// 2nd: dividing by total fractional area gets actual depth
	if (TotalPerviousCover_frac)     // This If statement is stepped into whenever Total != 0, ie is non-zero
	{
		// 1st: Summing 3 fractional area contributions
		// ThroughFlow of Pervious Area (value) = water under trees * fraction of watershed with trees on pervious area
		double waterOnPerviousAreaVal_m = folder->VarDict["waterUnderTree"] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
		// + water under shortveg * fraction of watershed with shortveg on pervious area
		waterOnPerviousAreaVal_m += folder->VarDict["waterUnderShortVeg"] * folder->ParamDict["ShortVegetationCover_frac"];
		// + water on open pervious area * fraction of waterhsed with bare soil (open pervious area)
		waterOnPerviousAreaVal_m += folder->VarDict["waterOpenArea"] * folder->ParamDict["SoilCover_frac"];

		// 2nd: Dividing by total fractional pervious area completes weighted average of through flow on pervious area
		waterOnPerviousAreaVal_m /= TotalPerviousCover_frac;

		//cte - runoffRouted undefined, model results are unchanged when this line is removed
		waterOnPerviousAreaVal_m += (folder->VarDict["runoffRouted"] / TotalPerviousCover_frac);

		// Storing waterOnPerviousAreaVal_m as VarDict["waterOnPerviousArea"], without impervious area runon
		//		Later in this function, will store waterOnPerviousAreaVal_m as a variable with impervious runon included
		folder->VarDict["waterOnPerviousArea"] = waterOnPerviousAreaVal_m;

		// Adding impRunoffToSoil_m, which had been multiplied by ImpAreaNotDirectlyConnected2Outlet_frac in ImperviousDepressionStorageCalc.cpp
		// Need to divide by TotalPerviousCover_frac to get depth on pervious area (m) 
		waterOnPerviousAreaVal_m += (folder->VarDict["impRunoffToSoil_m"] / TotalPerviousCover_frac);

		// Storing waterOnPerviousAreaVal_m as VarDict["waterToPerArea"], with impervious area runon included
		if (folder->ParamStringDict["Type"] == "BulkArea") {
			
			folder->VarDict["waterToPerDep"] = waterOnPerviousAreaVal_m;
		}
		else {
			//Combine waterOnPerviousAreaVal_m with runon from Bulk Area (BA) contributing area draining to GI 
			folder->VarDict["waterToPerDep_pre"] = waterOnPerviousAreaVal_m +
				input->RepoDict["impRunoffContArea_GI"] +
				input->RepoDict["infilExRunoffContArea_GI"] +
				input->RepoDict["satExRunoffContArea_GI"];	

			folder->VarDict["waterFromBA"] = input->RepoDict["impRunoffContArea"] +
				input->RepoDict["infilExRunoffContArea"] +
				input->RepoDict["satExRunoffContArea"];

		}
	}
	else  // If TotalPerviousCover_frac = 0, Throughflow on Pervious area = 0
		folder->VarDict["waterOnPerviousArea"] = 0.0;
    storeValues(input, folder, timeStep);

}

void WaterOnPerviousAreaCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	if (input->SimulationStringParams["Model"]=="StatisticalHydro"){
		folder->VarVecDict["waterToPerDep"].emplace_back(folder->VarDict["waterToPerDep"]);
		folder->VarVecDict["waterOnPerArea"].emplace_back(folder->VarDict["waterOnPerviousArea"]);
		folder->VarVecDict["impRunoffToSoil_adj"].emplace_back(folder->VarDict["impRunoffToSoil_m"] / folder->ParamDict["TotalPerviousCover_frac"]);
	}
}


