#include "GIOutputWriter.h"

void GIOutputWriter::writeVegCanopyFile_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m)"
		<< "," << "SWE_(m)"
		<< "," << "Intercept_TC_W_(m)"
		<< "," << "Intercept_TC_SWE_(m)"
		<< "," << "Evap_TC_W_(m)"
		<< "," << "Evap_TC_SWE_(m)"
		<< "," << "LeafStore_TC_W_(m)"
		<< "," << "LeafStore_TC_SWE_(m)"
		<< "," << "ThroughFall_TC_W_(m)"
		<< "," << "ThroughFall_TC_SWE_(m)"
		<< "," << "TreeCanopy_WB__(m)"
		<< "," << "Rain_(m)"
		<< "," << "SWE_(m)"
		<< "," << "Intercept_SV_W_(m)"
		<< "," << "Intercept_SW_SWE_(m)"
		<< "," << "Evap_SV_W_(m)"
		<< "," << "Evap_SV_SWE_(m)"
		<< "," << "LeafStore_SV_W_(m)"
		<< "," << "LeafStore_SV_SWE"
		<< "," << "ThroughFall_SV_W_(m)"
		<< "," << "ThroughFall_SV_SWE_(m)"
		<< "," << "ShortVegCanopy_WB_(m)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
         
		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}




		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["treeIntW"][i]
			<< "," << folder->VarVecDict["treeIntSWE"][i]
			<< "," << folder->VarVecDict["treeEvapW"][i]
			<< "," << folder->VarVecDict["treeEvapSWE"][i]
			<< "," << folder->VarVecDict["treeStoredW"][i]
			<< "," << folder->VarVecDict["treeStoredSWE"][i]
			<< "," << folder->VarVecDict["treeThroughW"][i]
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i]
			<< "," << tree_balance
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["shortVegIntW"][i]
			<< "," << folder->VarVecDict["shortVegIntSWE"][i]
			<< "," << folder->VarVecDict["shortVegEvapW"][i]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i]
			<< "," << folder->VarVecDict["shortVegStoredW"][i]
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i]
			<< "," << folder->VarVecDict["shortVegThroughW"][i]
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i]
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeVegCanopyFile_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;

	double TotalTreeCover_m2 = folder->ParamDict["TotalTreeCover"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m3)"
		<< "," << "SWE_(m3)"
		<< "," << "Rain_TC_(m3)"
		<< "," << "SWE_TC_(m3)"
		<< "," << "Intercept_TC_W_(m3)"
		<< "," << "Intercept_TC_W_(m3)"
		<< "," << "Evap_TC_W_(m3)"
		<< "," << "Evap_TC_SWE_(m3)"
		<< "," << "LeafStore_TC_W_(m3)"
		<< "," << "LeafStore_TC_SWE_(m3)"
		<< "," << "ThroughFall_TC_W_(m3)"
		<< "," << "ThroughFall_TC_SWE_(m3)"
		<< "," << "TreeCanopy_WB__(m3)"
		<< "," << "Rain_SV_(m3)"
		<< "," << "SWE_SV_(m3)"
		<< "," << "Intercept_SV_W_(m3)"
		<< "," << "Intercept_SW_SWE_(m3)"
		<< "," << "Evap_SV_W_(m3)"
		<< "," << "Evap_SV_SWE_(m3)"
		<< "," << "LeafStore_SV_W_(m3)"
		<< "," << "LeafStore_SV_SWE"
		<< "," << "ThroughFall_SV_W_(m3)"
		<< "," << "ThroughFall_SV_W_(m3)"
		<< "," << "ShortVegCanopy_WB_(m3)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				(0) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				(0) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				((folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				((folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * TotalTreeCover_m2
			<< "," << input->Snow[i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TotalTreeCover_m2
			<< "," << tree_balance
			<< "," << input->Rain[i] * ShortVegCover_m2
			<< "," << input->Snow[i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeWaterOnPerAreaFile_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterOnPerAreaFile_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	double TreePerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	double ShortVegCover_frac = folder->ParamDict["ShortVegetationCover_frac"];
	double BareSoilCover_frac = folder->ParamDict["SoilCover_frac"];
	double TotalPerviousCover_frac = BareSoilCover_frac + ShortVegCover_frac + TreePerviousCover_frac;

	double PerArea_WB = 0.0;
	double PerArea_SWEB = 0.0;
	double PerArea_comb_WB = 0.0;
	double PerArea_Flow_To_Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m)"
		<< "," << "Rain_BS_(m)"
		<< "," << "ThroughFlow_TCPA_W_(m)"
		<< "," << "ThroughFlow_SV_W_(m)"
		<< "," << "SnowMelt_TCPA_(m)"
		<< "," << "SnowMelt_SV_(m)"
		<< "," << "SnowMelt_BS_(m)"
		<< "," << "Water_PA_(m)"
		<< "," << "PerArea_WB_(m)"
		<< "," << "SWE_(m)"
		<< "," << "SWE_BareSoil_(m)"
		<< "," << "ThroughFlow_TCPA_SWE_(m)"
		<< "," << "ThroughFlow_SV_SWE_(m)"
		<< "," << "SWE_on_TCPA_(m)"
		<< "," << "SWE_on_SV_(m)"
		<< "," << "SWE_on_BS_(m)"
		<< "," << "SnowSub_TCPA_(m)"
		<< "," << "SnowSub_SV_(m)"
		<< "," << "SnowSub_BS_(m)"
		<< "," << "PerArea_SWEB_(m)"
		<< "," << "PerArea_comb_WB_(m)"
		<< "," << "Water_PA_(m)"
		<< "," << "Run_on_from_IA_(m)"
		<< "," << "Inflow_to_PerDep_(m)"
		<< "," << "PerDepStor_PA_(m)"
		<< "," << "PerDepEvap_PA_(m)"
		<< "," << "PerFlow_To_Infil_(m)"
		<< "," << "PA_Flow_To_Infil_WB_(m)"
		<< std::endl;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		if (i == 0)
		{
			PerArea_WB = (input->Rain[i]  * BareSoilCover_frac) +									// added rain to BS
				(folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_frac) +									// added rain to TCPA
				(folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_frac) +									// added rain to SV
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac) +									// Added snowmelt to BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac) +								// Added snowmelt to TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac) - 								// Added snowmelt to SV
				(folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac);								// Final water on IA - From model, includes snowmelt - have to unweight to match

			PerArea_SWEB = (0) +															// previously stored SWE BS
				(0) +																		// previously stored SWE TCPA
				(0) +																		// previously stored SWE SV
				(input->Snow[i] * BareSoilCover_frac) +													// added SWE to BS
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_frac) + 										// added SWE to TCPA
				(folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_frac) + 										// added SWE to SV
				(folder->VarVecDict["snowOpen"][i] * BareSoilCover_frac) -												// current stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_frac) -										// current stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_frac) -										// current stored SWE TCPA
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac) -											// Loss snowmelt from BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac) -										// Loss snowmelt from TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac) -										// Loss snowmelt from SV
				(folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_frac) -											// Loss of SnowSub BS
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_frac) -										// Loss of SnowSub TCPA
				(folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_frac);										// Loss of SnowSub TSV

			PerArea_comb_WB = PerArea_WB + PerArea_SWEB;											// combined WB

			PerArea_Flow_To_Infil_WB = (folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac) +											// Incoming water only (rain + snowmelt) - already weighted
				folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_frac +	//cte - vs impRunoffToSoil, check if multiplier needed, change entire .cpp																// Added water from impervious run-on
				(0) - 																				// Previous PerDep sTorage
				(folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_frac) -																		// Loss to Dep Evap
				(folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_frac) -																	// Current PerDep Stor
				(folder->VarVecDict["runoffToInfil"][i] * TotalPerviousCover_frac);																	// flow to infiltraiton routine
		}
		else
		{

			PerArea_WB = (input->Rain[i] * BareSoilCover_frac) +									// added rain to BS
				(folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_frac) +									// added rain to TCPA
				(folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_frac) +									// added rain to SV
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac) +									// Added snowmelt to BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac) +								// Added snowmelt to TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac) - 								// Added snowmelt to SV
				(folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac);								// Final water on IA - From model, includes snowmelt - have to unweight to match

			PerArea_SWEB = (folder->VarVecDict["snowOpen"][i-1] * BareSoilCover_frac) +															// previously stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i-1] * TreePerviousCover_frac) +																		// previously stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i-1] * ShortVegCover_frac) +																		// previously stored SWE SV
				(input->Snow[i] * BareSoilCover_frac) +													// added SWE to BS
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_frac) + 										// added SWE to TCPA
				(folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_frac) - 										// added SWE to SV
				(folder->VarVecDict["snowOpen"][i] * BareSoilCover_frac) -												// current stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_frac) -										// current stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_frac) -										// current stored SWE TCPA
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac) -											// Loss snowmelt from BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac) -										// Loss snowmelt from TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac) -										// Loss snowmelt from SV
				(folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_frac) -											// Loss of SnowSub BS
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_frac) -										// Loss of SnowSub TCPA
				(folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_frac);										// Loss of SnowSub TSV

			PerArea_comb_WB = PerArea_WB + PerArea_SWEB;											// combined WB

			PerArea_Flow_To_Infil_WB = (folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac) +											// Incoming water only (rain + snowmelt) - already weighted
				folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_frac +	//cte - check if we need fraction multiplier																// Added water from impervious run-on
				(folder->VarVecDict["perDepStor"][i-1] * TotalPerviousCover_frac) - 																	// Previous PerDep sTorage
				(folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_frac) -																		// Loss to Dep Evap
				(folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_frac) -																		// Current PerDep Stor
				(folder->VarVecDict["runoffToInfil"][i] * TotalPerviousCover_frac);																	// flow to infiltraiton routine
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Rain[i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_frac
			<< "," << folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac
			<< "," << folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac
			<< "," << PerArea_WB
			<< "," << input->Snow[i]
			<< "," << input->Snow[i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_frac
			<< "," << folder->VarVecDict["snowOpen"][i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_frac
			<< "," << folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_frac
			<< "," << PerArea_SWEB
			<< "," << PerArea_comb_WB
			<< "," << folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac
			<< "," << folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_frac
			<< "," << folder->VarVecDict["waterToPerDep"][i] * TotalPerviousCover_frac	
			<< "," << folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_frac					//Time series, use instantaneous value, or compare between two time steps, should not be summed in analysis
			<< "," << folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_frac
			<< "," << folder->VarVecDict["runoffToInfil"][i] * TotalPerviousCover_frac				// already adjusted by the fraction PerArea 
			<< "," << PerArea_Flow_To_Infil_WB
			<< std::endl;
	}

}

void GIOutputWriter::writeWaterOnPerAreaFile_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterOnPerAreaFile_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	double TreePerviousCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double BareSoilCover_m2 = folder->ParamDict["SoilCover_frac"] * folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = TreePerviousCover_m2 + ShortVegCover_m2 + BareSoilCover_m2;

	double PerArea_WB = 0.0;
	double PerArea_SWEB = 0.0;
	double PerArea_comb_WB = 0.0;
	double PerArea_Flow_To_Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m3)"
		<< "," << "Rain_BS_(m3)"
		<< "," << "ThroughFlow_TCPA_W_(m3)"
		<< "," << "ThroughFlow_SV_W_(m3)"
		<< "," << "SnowMelt_TCPA_(m3)"
		<< "," << "SnowMelt_SV_(m3)"
		<< "," << "SnowMelt_BS_(m3)"
		<< "," << "Water_PA_(m3)"
		<< "," << "PerArea_WB_(m3)"
		<< "," << "SWE_(m3)"
		<< "," << "SWE_BareSoil_(m3)"
		<< "," << "ThroughFlow_TCPA_SWE_(m3)"
		<< "," << "ThroughFlow_SV_SWE_(m3)"
		<< "," << "SWE_on_TCPA_(m3)"
		<< "," << "SWE_on_SV_(m3)"
		<< "," << "SWE_on_BS_(m3)"
		<< "," << "SnowSub_TCPA_(m3)"
		<< "," << "SnowSub_SV_(m3)"
		<< "," << "SnowSub_BS_(m3)"
		<< "," << "PerArea_SWEB_(m3)"
		<< "," << "PerArea_comb_WB_(m3)"
		<< "," << "Water_PA_Adj_(m3)"
		<< "," << "Run_on_from_IA_(m3)"
		<< "," << "Inflow_to_PerDep_(m3)"
		<< "," << "PerDepStor_PA_(m3)"
		<< "," << "PerDepEvap_PA_(m3)"
		<< "," << "PerFlow_To_Infil_(m3)"
		<< "," << "PA_Flow_To_Infil_WB_(m3)"
		<< std::endl;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		if (i == 0)
		{
			PerArea_WB = (input->Rain[i]  * BareSoilCover_m2) +									// added rain to BS
				(folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_m2) +									// added rain to TCPA
				(folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2) +									// added rain to SV
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2) +									// Added snowmelt to BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2) +								// Added snowmelt to TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2) - 								// Added snowmelt to SV
				(folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2);								// Final water on IA - From model, includes snowmelt - have to unweight to match

			PerArea_SWEB = (0) +															// previously stored SWE BS
				(0) +																		// previously stored SWE TCPA
				(0) +																		// previously stored SWE SV
				(input->Snow[i] * BareSoilCover_m2) +													// added SWE to BS
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_m2) + 										// added SWE to TCPA
				(folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2) + 										// added SWE to SV
				(folder->VarVecDict["snowOpen"][i] * BareSoilCover_m2) -												// current stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_m2) -										// current stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_m2) -										// current stored SWE TCPA
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2) -											// Loss snowmelt from BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2) -										// Loss snowmelt from TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2) -										// Loss snowmelt from SV
				(folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_m2) -											// Loss of SnowSub BS
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_m2) -										// Loss of SnowSub TCPA
				(folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_m2);										// Loss of SnowSub TSV

			PerArea_comb_WB = PerArea_WB + PerArea_SWEB;											// combined WB

			PerArea_Flow_To_Infil_WB = folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2 +											// Incoming water only (rain + snowmelt) - already weighted
				folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_m2 +																	// Added water from impervious run-on
				(0) - 																				// Previous PerDep sTorage
				folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_m2 -																		// Loss to Dep Evap
				folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_m2 -																	// Current PerDep Stor
				folder->VarVecDict["runoffToInfil"][i] * TotalPerviousCover_m2;																	// flow to infiltraiton routine
		}
		else
		{

			PerArea_WB = (input->Rain[i] * BareSoilCover_m2) +									// added rain to BS
				(folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_m2) +									// added rain to TCPA
				(folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2) +									// added rain to SV
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2) +									// Added snowmelt to BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2) +								// Added snowmelt to TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2) - 								// Added snowmelt to SV
				(folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2);								// Final water on IA - From model, includes snowmelt - have to unweight to match

			PerArea_SWEB = (folder->VarVecDict["snowOpen"][i-1] * BareSoilCover_m2) +															// previously stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i-1] * TreePerviousCover_m2) +																		// previously stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i-1] * ShortVegCover_m2) +																		// previously stored SWE SV
				(input->Snow[i] * BareSoilCover_m2) +													// added SWE to BS
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_m2) + 										// added SWE to TCPA
				(folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2) - 										// added SWE to SV
				(folder->VarVecDict["snowOpen"][i] * BareSoilCover_m2) -												// current stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_m2) -										// current stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_m2) -										// current stored SWE TCPA
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2) -											// Loss snowmelt from BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2) -										// Loss snowmelt from TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2) -										// Loss snowmelt from SV
				(folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_m2) -											// Loss of SnowSub BS
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_m2) -										// Loss of SnowSub TCPA
				(folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_m2);										// Loss of SnowSub TSV

			PerArea_comb_WB = PerArea_WB + PerArea_SWEB;											// combined WB

			PerArea_Flow_To_Infil_WB = folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2 +											// Incoming water only (rain + snowmelt) - already weighted
				folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_m2 +																	// Added water from impervious run-on
				folder->VarVecDict["perDepStor"][i-1] * TotalPerviousCover_m2 - 																	// Previous PerDep sTorage
				folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_m2 -																		// Loss to Dep Evap
				folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_m2 -																		// Current PerDep Stor
				folder->VarVecDict["runoffToInfil"][i] * TotalPerviousCover_m2;																	// flow to infiltraiton routine
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2
			<< "," << PerArea_WB
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["snowOpen"][i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_m2
			<< "," << PerArea_SWEB
			<< "," << PerArea_comb_WB
			<< "," << folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["waterToPerDep"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_m2					//Time series, use instantaneous value, or compare between two time steps, should not be summed in analysis
			<< "," << folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["runoffToInfil"][i] * TotalPerviousCover_m2				// already adjusted by the fraction PerArea 
			<< "," << PerArea_Flow_To_Infil_WB
			<< std::endl;
	}

}

void GIOutputWriter::writeWaterOnImpAreaFile_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterOnImpAreaFile_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	double TreeImperviousCover_frac = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
	double NoTreeImperviousCover_frac = folder->ParamDict["ImperviousCover_frac"];
	double TotalImperviousCover_frac = TreeImperviousCover_frac + NoTreeImperviousCover_frac;

	double ImpArea_WB = 0.0;
	double ImpArea_SWEB = 0.0;
	double ImpArea_comb_WB = 0.0;
	double ImpArea_Runoff_WB = 0.0;
	double total_IA_W_SWE = 0.0;
	double in = 0.0;
	double out = 0.0;
	double loss = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_OIA_(m)"
		<< "," << "ThroughFlow_Tree_W_TCIA_(m)"
		<< "," << "SnowMelt_OIA_(m)"
		<< "," << "SnowMelt_TCIA_(m)"
		<< "," << "Water_IA_(m)"
		<< "," << "ImpArea_WB_(m)"
		<< "," << "SWE_OIA_(m)"
		<< "," << "ThroughFlow_Tree_SWE_TCIA_(m)"
		<< "," << "SWE_on_OIA_(m)"
		<< "," << "SWE_on_TCIA_(m)"
		<< "," << "SnowSub_OIA_(m)"
		<< "," << "SnowSub_TCIA_(m)"
		<< "," << "ImpArea_SWEB_(m)"
		<< "," << "ImpArea_comb_WB_(m)"
		<< "," << "ImpDepStor_IA_(m)"
		<< "," << "ImpDepEvap_IA_(m)"
		<< "," << "ImperFlow_Soil_(m)"
		<< "," << "ImperFlow_Outlet_nonRouted_(m)"
		<< "," << "ImpArea_Runoff_WB_(m)"
		<< std::endl;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		if (i == 0)
		{

			ImpArea_WB = (input->Rain[i] * NoTreeImperviousCover_frac) +									// added rain to OIA
				(folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_frac) +									// added rain to TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac) +								// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac) -								// Added snowmelt to TCIA
				(folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac);		// Final water on IA - From model, includes snowmelt

			ImpArea_SWEB = (0) +																				// previously stored SWE OIA
				(0) +																							// previously stored SWE TCIA
				(input->Snow[i] * NoTreeImperviousCover_frac) +											// added SWE to OIA
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_frac) - 					// added SWE to TCIA
				(folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_frac) -									// current stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_frac) -									// current stored SWE TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac) -								// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac) -								// Loss snowmelt from TCIA
				(folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_frac) -								// Loss of SnowSub OIA
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_frac);								// Loss of SnowSub TCIA

			ImpArea_comb_WB = ImpArea_WB + ImpArea_SWEB;								// combined WB

			ImpArea_Runoff_WB = (folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac) +		// Incoming water only (rain + snowmelt) 
				(0) - 																											// Previous ImpDep sTorage
				(folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_frac) -							// Loss to Dep Evap
				(folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_frac) -							// Current ImpDep Stor
				folder->VarVecDict["impRunoffToSoil"][i] -																			// loss to run on to pervious area
				folder->VarVecDict["impRunoffDCIA"][i];																			// loss to DCIA flow to outlet
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Sub - predicted Water on IA - current stored = 0 
			ImpArea_WB = (input->Rain[i] * NoTreeImperviousCover_frac) +										// added rain to OIA
				(folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_frac) +										// added rain to TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac) +									// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac) -									// Added snowmelt to TCIA
				(folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac);			// Final water on IA - From model, includes snowmelt

			ImpArea_SWEB = (folder->VarVecDict["snowOpen"][i-1] * NoTreeImperviousCover_frac) +							// previously stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i-1] * TreeImperviousCover_frac) +									// previously stored SWE TCIA
				(input->Snow[i] * NoTreeImperviousCover_frac) +												// added SWE to OIA
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_frac) - 									// added SWE to TCIA
				(folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_frac) -										// current stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_frac) -										// current stored SWE TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac) -									// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac) -									// Loss snowmelt from TCIA
				(folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_frac) -									// Loss of SnowSub OIA
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_frac);									// Loss of SnowSub TCIA

			ImpArea_comb_WB = ImpArea_WB + ImpArea_SWEB;								// combined WB

			ImpArea_Runoff_WB = (folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac) +		// Incoming water only (rain + snowmelt) 
				(folder->VarVecDict["impDepStor"][i-1] * TotalImperviousCover_frac) - 							// Previous ImpDep sTorage
				(folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_frac) -							// Loss to Dep Evap
				(folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_frac) -							// Current ImpDep Stor
				folder->VarVecDict["impRunoffToSoil"][i] -																	// loss to run on to pervious area
				folder->VarVecDict["impRunoffDCIA"][i];																	// loss to DCIa flow to outlet
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac
			<< "," << folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac
			<< "," << ImpArea_WB
			<< "," << input->Snow[i]  * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_frac
			<< "," << ImpArea_SWEB
			<< "," << ImpArea_comb_WB
			<< "," << folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_frac
			<< "," << folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_frac
			<< "," << folder->VarVecDict["impRunoffToSoil"][i]					//cte - check if already adjusted by the fraction ImpArea non-DCIA
			<< "," << folder->VarVecDict["impRunoffDCIA"][i]					// already adjusted by the fraction of ImpAreaDCIA
			<< "," << ImpArea_Runoff_WB
			<< std::endl;
	}


}

void GIOutputWriter::writeWaterOnImpAreaFile_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterOnImpAreaFile_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	double TreeImperviousCover_m2 = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"]	* folder->ParamDict["Area"];
	double NoTreeImperviousCover_m2 = folder->ParamDict["ImperviousCover_frac"] * folder->ParamDict["Area"];
	double TotalImperviousCover_m2 = TreeImperviousCover_m2 + NoTreeImperviousCover_m2;

	double ImpArea_WB = 0.0;
	double ImpArea_SWEB = 0.0;
	double ImpArea_comb_WB = 0.0;
	double ImpArea_Runoff_WB = 0.0;
	double total_IA_W_SWE = 0.0;
	double in = 0.0;
	double out = 0.0;
	double loss = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_OIA_(m3)"
		<< "," << "ThroughFlow_TCIA_W_(m3)"
		<< "," << "SnowMelt_OIA_(m3)"
		<< "," << "SnowMelt_TCIA_(m3)"
		<< "," << "Water_IA_(m3)"
		<< "," << "ImpArea_WB_(m3)"
		<< "," << "SWE_OIA_(m3)"
		<< "," << "ThroughFlow_TCIA_SWE_(m3)"
		<< "," << "SWE_on_OIA_(m3)"
		<< "," << "SWE_on_TCIA_(m3)"
		<< "," << "SnowSub_OIA_(m3)"
		<< "," << "SnowSub_TCIA_(m3)"
		<< "," << "ImpArea_SWEB_(m3)"
		<< "," << "ImpArea_comb_WB_(m3)"
		<< "," << "ImpDepStor_IA_(m3)"
		<< "," << "ImpDepEvap_IA_(m3)"
		<< "," << "ImperFlow_Soil_(m3)"
		<< "," << "ImperFlow_Outlet_nonRouted_(m3)"
		<< "," << "ImpArea_Runoff_WB_(m3)"
		<< std::endl;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		if (i == 0)
		{

			ImpArea_WB = (input->Rain[i] * NoTreeImperviousCover_m2) +									// added rain to OIA
				(folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_m2) +									// added rain to TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2) +								// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2) -								// Added snowmelt to TCIA
				(folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2);		// Final water on IA - From model, includes snowmelt

			ImpArea_SWEB = (0) +																				// previously stored SWE OIA
				(0) +																							// previously stored SWE TCIA
				(input->Snow[i] * NoTreeImperviousCover_m2) +											// added SWE to OIA
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_m2) - 								// added SWE to TCIA
				(folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_m2) -									// current stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_m2) -									// current stored SWE TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2) -								// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2) -								// Loss snowmelt from TCIA
				(folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_m2) -								// Loss of SnowSub OIA
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_m2);								// Loss of SnowSub TCIA

			ImpArea_comb_WB = ImpArea_WB + ImpArea_SWEB;								// combined WB

			ImpArea_Runoff_WB = (folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2) +		// Incoming water only (rain + snowmelt) 
				(0) - 																											// Previous ImpDep sTorage
				(folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_m2) -							// Loss to Dep Evap
				(folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_m2) -							// Current ImpDep Stor
				(folder->VarVecDict["impRunoffToSoil"][i] * folder->ParamDict["Area"]) -				//cte - check if Area or TIC_f loss to run on to pervious area
				(folder->VarVecDict["impRunoffDCIA"][i] * folder->ParamDict["Area"]);					// loss to DCIA flow to outlet
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Sub - predicted Water on IA - current stored = 0 
			ImpArea_WB = (input->Rain[i] * NoTreeImperviousCover_m2) +										// added rain to OIA
				(folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_m2) +										// added rain to TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2) +									// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2) -									// Added snowmelt to TCIA
				(folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2);			// Final water on IA - From model, includes snowmelt

			ImpArea_SWEB = (folder->VarVecDict["snowOpen"][i-1] * NoTreeImperviousCover_m2) +							// previously stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i-1] * TreeImperviousCover_m2) +									// previously stored SWE TCIA
				(input->Snow[i] * NoTreeImperviousCover_m2) +												// added SWE to OIA
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_m2) - 									// added SWE to TCIA
				(folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_m2) -										// current stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_m2) -										// current stored SWE TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2) -									// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2) -									// Loss snowmelt from TCIA
				(folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_m2) -									// Loss of SnowSub OIA
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_m2);									// Loss of SnowSub TCIA

			ImpArea_comb_WB = ImpArea_WB + ImpArea_SWEB;								// combined WB

			ImpArea_Runoff_WB = (folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2) +		// Incoming water only (rain + snowmelt) 
				(folder->VarVecDict["impDepStor"][i-1] * TotalImperviousCover_m2) - 							// Previous ImpDep sTorage
				(folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_m2) -							// Loss to Dep Evap
				(folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_m2) -							// Current ImpDep Stor
				(folder->VarVecDict["impRunoffToSoil"][i] * folder->ParamDict["Area"]) -					//cte - check if Area or other_Frac loss to run on to pervious area
				(folder->VarVecDict["impRunoffDCIA"][i] * folder->ParamDict["Area"]);						// loss to DCIa flow to outlet
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2
			<< "," << folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2
			<< "," << ImpArea_WB
			<< "," << input->Snow[i]  * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_m2
			<< "," << ImpArea_SWEB
			<< "," << ImpArea_comb_WB
			<< "," << folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_m2
			<< "," << folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_m2
			<< "," << folder->VarVecDict["impRunoffToSoil"][i] * folder->ParamDict["Area"]					//cte- check if adjusted; already adjusted by the fraction ImpArea non-DCIA
			<< "," << folder->VarVecDict["impRunoffDCIA"][i] * folder->ParamDict["Area"]					// already adjusted by the fraction of ImpAreaDCIA
			<< "," << ImpArea_Runoff_WB
			<< std::endl;
	}


}

void GIOutputWriter::writeEvap_ET_Infil_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Evap_TC_W_(m)"
		<< "," << "Evap_TC_SWE_(m)"
		<< "," << "Evap_SV_W_(m)"
		<< "," << "Evap_SV_SWE_(m)"
		<< "," << "Infiltration_PA_(m)"
		<< "," << "rET_VegCover_(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * folder->ParamDict["TotalTreeCover"]
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * folder->ParamDict["TotalTreeCover"]
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["infilWeighted"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< "," << folder->VarVecDict["rET"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< std::endl;
	}
	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	double TotalTreeCover_m2 = folder->ParamDict["TotalTreeCover"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = folder->ParamDict["TotalPerviousCover_frac"] * folder->ParamDict["Area"];

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Evap_TC_W_(m3)"
		<< "," << "Evap_TC_SWE_(m3)"
		<< "," << "Evap_SV_W_(m3)"
		<< "," << "Evap_SV_SWE_(m3)"
		<< "," << "Infiltration_PA_(m3)"
		<< "," << "rET_VegCover_(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["infilWeighted"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["rET"][i] * TotalPerviousCover_m2
			<< std::endl;

	}
	outfile.close();
}

void GIOutputWriter::writeWaterInRootZone_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterInRootZone_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double RootZone_WB = 0.0;
	double perAreaRunoff_WB = 0.0;
	double Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "PA_Flow_To_Infil_(m)"
		<< "," << "Infil_InfilExcess_(m)"
		<< "," << "Infil_SatExcess_(m)"
		<< "," << "Infil_Weighted_Total_(m)"
		<< "," << "Infil_WB_(m)"
		<< "," << "Runoff_InfilExcess_(m)"
		<< "," << "Runoff_SatExcess_(m)"
		<< "," << "RootZone_WB_(m)"
		<< "," << "Total_PA_Runoff_(m)"
		<< "," << "PA_Runoff_WB(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		Infil_WB = folder->VarVecDict["infilWeighted"][i] -
			folder->VarVecDict["satExInfil"][i] -
			folder->VarVecDict["adjInfilExInfil"][i];


		RootZone_WB = folder->VarVecDict["runoffToInfil_postPMacro"][i] -
			(folder->VarVecDict["infilWeighted"][i] - folder->VarVecDict["satExRunoff"][i]) -
			(folder->VarVecDict["satExRunoff"][i] /** fracSatExcess*/) -
			(folder->VarVecDict["adjInfilExRunoff"][i]);

		perAreaRunoff_WB = (folder->VarVecDict["satExRunoff"][i] /** fracSatExcess*/) +
			(folder->VarVecDict["adjInfilExRunoff"][i] /** fracInfilExcess*/) -
			folder->VarVecDict["perAreaFlow_unAdj"][i];

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["runoffToInfil_postPMacro"][i]
			<< "," << folder->VarVecDict["adjInfilExInfil"][i]										// already weighted
			<< "," << folder->VarVecDict["satExInfil"][i]										// already weighted
			<< "," << folder->VarVecDict["infilWeighted"][i]								// already weighted
			<< "," << Infil_WB
			<< "," << folder->VarVecDict["adjInfilExRunoff"][i] //* fracInfilExcess
			<< "," << folder->VarVecDict["satExRunoff"][i] //* fracSatExcess    All of the soil contributes to saturation excess after the inital inflow constraint
			<< "," << RootZone_WB
			<< "," << folder->VarVecDict["perAreaFlow_unAdj"][i]
			<< "," << perAreaRunoff_WB
			<< std::endl;

	}
	outfile.close();

}

void GIOutputWriter::writeWaterInRootZone_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterInRootZone_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	double TotalPerviousCover_m2 = folder->ParamDict["TotalPerviousCover_frac"] * folder->ParamDict["Area"];

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double RootZone_WB = 0.0;
	double perAreaRunoff_WB = 0.0;
	double Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "PA_Flow_To_Infil_(m3)"
		<< "," << "Infil_InfilExcess_(m3)"
		<< "," << "Infil_SatExcess_(m3)"
		<< "," << "Infil_Weighted_Total_(m3)"
		<< "," << "Infil_WB_(m3)"
		<< "," << "Runoff_InfilExcess_(m3)"
		<< "," << "Runoff_SatExcess_(m3)"
		<< "," << "RootZone_WB_(m3)"
		<< "," << "Total_PA_Runoff_(m3)"
		<< "," << "PA_Runoff_WB(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		Infil_WB = (folder->VarVecDict["infilWeighted"][i] * TotalPerviousCover_m2) -
			(folder->VarVecDict["satExInfil"][i] * TotalPerviousCover_m2) -
			(folder->VarVecDict["adjInfilExInfil"][i] * TotalPerviousCover_m2);


		RootZone_WB = (folder->VarVecDict["runoffToInfil_postPMacro"][i] * TotalPerviousCover_m2) -
			((folder->VarVecDict["infilWeighted"][i] - folder->VarVecDict["satExRunoff"][i])* TotalPerviousCover_m2) -
			(folder->VarVecDict["satExRunoff"][i] * TotalPerviousCover_m2  /** fracSatExcess*/) -
			(folder->VarVecDict["adjInfilExRunoff"][i] * TotalPerviousCover_m2);

		perAreaRunoff_WB = (folder->VarVecDict["satExRunoff"][i] * TotalPerviousCover_m2 /** fracSatExcess*/) +
			(folder->VarVecDict["adjInfilExRunoff"][i] * TotalPerviousCover_m2 /** fracInfilExcess*/) -
			(folder->VarVecDict["perAreaFlow_unAdj"][i] * TotalPerviousCover_m2);

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["runoffToInfil_postPMacro"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["adjInfilExInfil"][i] * TotalPerviousCover_m2										// already weighted
			<< "," << folder->VarVecDict["satExInfil"][i] * TotalPerviousCover_m2										// already weighted
			<< "," << folder->VarVecDict["infilWeighted"][i] * TotalPerviousCover_m2								// already weighted
			<< "," << Infil_WB
			<< "," << folder->VarVecDict["adjInfilExRunoff"][i] * TotalPerviousCover_m2 //* fracInfilExcess
			<< "," << folder->VarVecDict["satExRunoff"][i] * TotalPerviousCover_m2 //* fracSatExcess    All of the soil contributes to saturation excess after the inital inflow constraint
			<< "," << RootZone_WB
			<< "," << folder->VarVecDict["perAreaFlow_unAdj"][i] * TotalPerviousCover_m2
			<< "," << perAreaRunoff_WB
			<< std::endl;

	}
	outfile.close();
}



void GIOutputWriter::writeWaterBalance_RainGarden(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_RainGarden.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Inflow(q0)"
		<< "," << "SurfaceInfil(f1)"
		<< "," << "Percolation(f2)"
		<< "," << "StorageExfil(f3)"
		<< "," << "SurfaceOutflow(q1)"
		<< "," << "StorageDrain(q3)"
		<< "," << "paveDepth(d4)"
		<< "," << "surfaceDepth(d1)"
		<< "," << "soilMoisture(d2)"
		<< "," << "storageDepth(d3)"
		//Added by Li to show the total Evap
		<< "," << "totalEvap"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["waterBalanceInflow"][i]
			<< "," << folder->VarVecDict["waterBalanceInfil"][i]
			<< "," << folder->VarVecDict["soilPerc"][i]
			<< "," << folder->VarVecDict["waterBalanceExfil"][i]
			<< "," << folder->VarVecDict["waterBalanceSurFlow"][i]
			<< "," << folder->VarVecDict["waterBalanceDrainFlow"][i]
			<< "," << folder->VarVecDict["paveDepth_out"][i]
			<< "," << folder->VarVecDict["surfaceDepth_out"][i]
			<< "," << folder->VarVecDict["soilMoisture_out"][i]
			<< "," << folder->VarVecDict["storageDepth_out"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["evaporation_all"][i]
			<< endl;
	}

	outfile.close();

}


void GIOutputWriter::writeWaterBalance_BioRetention(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_BioRetention.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Inflow(q0)"
		<< "," << "SurfaceInfil(f1)"
		<< "," << "Percolation(f2)"
		<< "," << "StorageExfil(f3)"
		<< "," << "SurfaceOutflow(q1)"
		<< "," << "StorageDrain(q3)"
		<< "," << "paveDepth(d4)"
		<< "," << "surfaceDepth(d1)"
		<< "," << "soilMoisture(d2)"
		<< "," << "storageDepth(d3)"
		//Added by Li to show the total Evap
		<< "," << "totalEvap"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["waterBalanceInflow"][i]
			<< "," << folder->VarVecDict["waterBalanceInfil"][i]
			<< "," << folder->VarVecDict["soilPerc"][i]
			<< "," << folder->VarVecDict["waterBalanceExfil"][i]
			<< "," << folder->VarVecDict["waterBalanceSurFlow"][i]
			<< "," << folder->VarVecDict["waterBalanceDrainFlow"][i]
			<< "," << folder->VarVecDict["paveDepth_out"][i]
			<< "," << folder->VarVecDict["surfaceDepth_out"][i]
			<< "," << folder->VarVecDict["soilMoisture_out"][i]
			<< "," << folder->VarVecDict["storageDepth_out"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["evaporation_all"][i]
			<< endl;
	}

	outfile.close();

}


void GIOutputWriter::writeWaterBalance_InfilTrench(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_InfilTrench.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Inflow(q0)"
		<< "," << "SurfaceInfil(f1)"
		<< "," << "Percolation(f2)"
		<< "," << "StorageExfil(f3)"
		<< "," << "SurfaceOutflow(q1)"
		<< "," << "StorageDrain(q3)"
		<< "," << "paveDepth(d4)"
		<< "," << "surfaceDepth(d1)"
		<< "," << "soilMoisture(d2)"
		<< "," << "storageDepth(d3)"
		//Added by Li to show the total Evap
		<< "," << "totalEvap"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["waterBalanceInflow"][i]
			<< "," << folder->VarVecDict["waterBalanceInfil"][i]
			<< "," << folder->VarVecDict["soilPerc"][i]
			<< "," << folder->VarVecDict["waterBalanceExfil"][i]
			<< "," << folder->VarVecDict["waterBalanceSurFlow"][i]
			<< "," << folder->VarVecDict["waterBalanceDrainFlow"][i]
			<< "," << folder->VarVecDict["paveDepth_out"][i]
			<< "," << folder->VarVecDict["surfaceDepth_out"][i]
			<< "," << folder->VarVecDict["soilMoisture_out"][i]
			<< "," << folder->VarVecDict["storageDepth_out"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["evaporation_all"][i]
			<< endl;
	}

	outfile.close();

}


void GIOutputWriter::writeWaterBalance_Swale(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_Swale.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Inflow(q0)"
		<< "," << "SurfaceInfil(f1)"
		<< "," << "Percolation(f2)"
		<< "," << "StorageExfil(f3)"
		<< "," << "SurfaceOutflow(q1)"
		<< "," << "StorageDrain(q3)"
		<< "," << "paveDepth(d4)"
		<< "," << "surfaceDepth(d1)"
		<< "," << "soilMoisture(d2)"
		<< "," << "storageDepth(d3)"
		//Added by Li to show the total Evap
		<< "," << "totalEvap"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["waterBalanceInflow"][i]
			<< "," << folder->VarVecDict["waterBalanceInfil"][i]
			<< "," << folder->VarVecDict["soilPerc"][i]
			<< "," << folder->VarVecDict["waterBalanceExfil"][i]
			<< "," << folder->VarVecDict["waterBalanceSurFlow"][i]
			<< "," << folder->VarVecDict["waterBalanceDrainFlow"][i]
			<< "," << folder->VarVecDict["paveDepth_out"][i]
			<< "," << folder->VarVecDict["surfaceDepth_out"][i]
			<< "," << folder->VarVecDict["soilMoisture_out"][i]
			<< "," << folder->VarVecDict["storageDepth_out"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["evaporation_all"][i]
			<< endl;
	}

	outfile.close();

}


void GIOutputWriter::writeWaterBalance_PermeablePavement(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_PermeablePavement.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Inflow(q0)"
		<< "," << "SurfaceInfil(f1)"
		<< "," << "Percolation(f2)"
		<< "," << "StorageExfil(f3)"
		<< "," << "SurfaceOutflow(q1)"
		<< "," << "StorageDrain(q3)"
		<< "," << "paveDepth(d4)"
		<< "," << "surfaceDepth(d1)"
		<< "," << "soilMoisture(d2)"
		<< "," << "storageDepth(d3)"
		<< "," << "pavePerc(f4)"
		//Added by Li to show the total Evap
		<< "," << "totalEvap"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["waterBalanceInflow"][i]
			<< "," << folder->VarVecDict["waterBalanceInfil"][i]
			<< "," << folder->VarVecDict["soilPerc"][i]
			<< "," << folder->VarVecDict["waterBalanceExfil"][i]
			<< "," << folder->VarVecDict["waterBalanceSurFlow"][i]
			<< "," << folder->VarVecDict["waterBalanceDrainFlow"][i]
			<< "," << folder->VarVecDict["paveDepth_out"][i]
			<< "," << folder->VarVecDict["surfaceDepth_out"][i]
			<< "," << folder->VarVecDict["soilMoisture_out"][i]
			<< "," << folder->VarVecDict["storageDepth_out"][i]
			<< "," << folder->VarVecDict["pavePerc_out"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["evaporation_all"][i]
			<< endl;
	}

	outfile.close();

}

//Li: add waterbalance output for green roof
void GIOutputWriter::writeWaterBalance_GreenRoof(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_GreenRoof.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Inflow(q0)"
		<< "," << "SurfaceInfil(f1)"
		<< "," << "Percolation(f2)"
		<< "," << "StorageExfil(f3)"
		<< "," << "SurfaceOutflow(q1)"
		<< "," << "StorageDrain(q3)"
		<< "," << "paveDepth(d4)"
		<< "," << "surfaceDepth(d1)"
		<< "," << "soilMoisture(d2)"
		<< "," << "storageDepth(d3)"
		//Added by Li to show the total Evap
		<< "," << "totalEvap"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["waterBalanceInflow"][i]
			<< "," << folder->VarVecDict["waterBalanceInfil"][i]
			<< "," << folder->VarVecDict["soilPerc"][i]
			<< "," << folder->VarVecDict["waterBalanceExfil"][i]
			<< "," << folder->VarVecDict["waterBalanceSurFlow"][i]
			<< "," << folder->VarVecDict["waterBalanceDrainFlow"][i]
			<< "," << folder->VarVecDict["paveDepth_out"][i]
			<< "," << folder->VarVecDict["surfaceDepth_out"][i]
			<< "," << folder->VarVecDict["soilMoisture_out"][i]
			<< "," << folder->VarVecDict["storageDepth_out"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["evaporation_all"][i]
			<< endl;
	}

	outfile.close();

}

//Li: added waterbalance output for rooftop disconnection
void GIOutputWriter::writeWaterBalance_RoofDiscon(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_RoofDiscon.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Inflow(q0)"
		<< "," << "SurfaceInfil(f1)"
		<< "," << "Percolation(f2)"
		<< "," << "StorageExfil(f3)"
		<< "," << "SurfaceOutflow(q1)"
		<< "," << "StorageDrain(q3)"
		<< "," << "paveDepth(d4)"
		<< "," << "surfaceDepth(d1)"
		<< "," << "soilMoisture(d2)"
		<< "," << "storageDepth(d3)"
		//Added by Li to show the total Evap
		<< "," << "totalEvap"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["waterBalanceInflow"][i]
			<< "," << folder->VarVecDict["waterBalanceInfil"][i]
			<< "," << folder->VarVecDict["soilPerc"][i]
			<< "," << folder->VarVecDict["waterBalanceExfil"][i]
			<< "," << folder->VarVecDict["waterBalanceSurFlow"][i]
			<< "," << folder->VarVecDict["waterBalanceDrainFlow"][i]
			<< "," << folder->VarVecDict["paveDepth_out"][i]
			<< "," << folder->VarVecDict["surfaceDepth_out"][i]
			<< "," << folder->VarVecDict["soilMoisture_out"][i]
			<< "," << folder->VarVecDict["storageDepth_out"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["evaporation_all"][i]
			<< endl;
	}

	outfile.close();

}

//Li:added waterbalance output for rain barrels
void GIOutputWriter::writeWaterBalance_RainBarrel(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_RainBarrel.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Inflow(q0)"
		<< "," << "SurfaceInfil(f1)"
		<< "," << "Percolation(f2)"
		<< "," << "StorageExfil(f3)"
		<< "," << "SurfaceOutflow(q1)"
		<< "," << "StorageDrain(q3)"
		<< "," << "paveDepth(d4)"
		<< "," << "surfaceDepth(d1)"
		<< "," << "soilMoisture(d2)"
		<< "," << "storageDepth(d3)"
		//Added by Li to show the total Evap
		<< "," << "totalEvap"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["waterBalanceInflow"][i]
			<< "," << folder->VarVecDict["waterBalanceInfil"][i]
			<< "," << folder->VarVecDict["soilPerc"][i]
			<< "," << folder->VarVecDict["waterBalanceExfil"][i]
			<< "," << folder->VarVecDict["waterBalanceSurFlow"][i]
			<< "," << folder->VarVecDict["waterBalanceDrainFlow"][i]
			<< "," << folder->VarVecDict["paveDepth_out"][i]
			<< "," << folder->VarVecDict["surfaceDepth_out"][i]
			<< "," << folder->VarVecDict["soilMoisture_out"][i]
			<< "," << folder->VarVecDict["storageDepth_out"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["evaporation_all"][i]
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeVegCanopyFile_VWB_RainGarden(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_VWB_RainGarden.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m)"
		<< "," << "SWE_(m)"
		<< "," << "RG_Intercept_TC_W_(m)"
		<< "," << "RG_Intercept_TC_SWE_(m)"
		<< "," << "RG_Evap_TC_W_(m)"
		<< "," << "RG_Evap_TC_SWE_(m)"
		<< "," << "RG_LeafStore_TC_W_(m)"
		<< "," << "RG_LeafStore_TC_SWE_(m)"
		<< "," << "RG_ThroughFall_TC_W_(m)"
		<< "," << "RG_ThroughFall_TC_SWE_(m)"
		<< "," << "RG_TreeCanopy_WB__(m)"
		<< "," << "RG_Rain_SV_(m)"
		<< "," << "RG_SWE_SV_(m)"
		<< "," << "RG_Intercept_SV_W_(m)"
		<< "," << "RG_Intercept_SW_SWE_(m)"
		<< "," << "RG_Evap_SV_W_(m)"
		<< "," << "RG_Evap_SV_SWE_(m)"
		<< "," << "RG_LeafStore_SV_W_(m)"
		<< "," << "RG_LeafStore_SV_SWE"
		<< "," << "RG_ThroughFall_SV_W_(m)"
		<< "," << "RG_ThroughFall_SV_SWE_(m)"
		<< "," << "RG_ShortVegCanopy_WB_(m)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}




		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["treeIntW"][i]
			<< "," << folder->VarVecDict["treeIntSWE"][i]
			<< "," << folder->VarVecDict["treeEvapW"][i]
			<< "," << folder->VarVecDict["treeEvapSWE"][i]
			<< "," << folder->VarVecDict["treeStoredW"][i]
			<< "," << folder->VarVecDict["treeStoredSWE"][i]
			<< "," << folder->VarVecDict["treeThroughW"][i]
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i]
			<< "," << tree_balance
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["shortVegIntW"][i]
			<< "," << folder->VarVecDict["shortVegIntSWE"][i]
			<< "," << folder->VarVecDict["shortVegEvapW"][i]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i]
			<< "," << folder->VarVecDict["shortVegStoredW"][i]
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i]
			<< "," << folder->VarVecDict["shortVegThroughW"][i]
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i]
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeVegCanopyFile_FWB_RainGarden(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_FWB_RainGarden.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m3)"
		<< "," << "SWE_(m3)"
		<< "," << "RG_Rain_TC_(m3)"
		<< "," << "RG_SWE_TC_(m3)"
		<< "," << "RG_Intercept_TC_W_(m3)"
		<< "," << "RG_Intercept_TC_SWE_(m3)"
		<< "," << "RG_Evap_TC_W_(m3)"
		<< "," << "RG_Evap_TC_SWE_(m3)"
		<< "," << "RG_LeafStore_TC_W_(m3)"
		<< "," << "RG_LeafStore_TC_SWE_(m3)"
		<< "," << "RG_ThroughFall_TC_W_(m3)"
		<< "," << "RG_ThroughFall_TC_SWE_(m3)"
		<< "," << "RG_TreeCanopy_WB_(m3)"
		<< "," << "RG_Rain_SV_(m3)"
		<< "," << "RG_SWE_SV_(m3)"
		<< "," << "RG_Intercept_SV_W_(m3)"
		<< "," << "RG_Intercept_SW_SWE_(m3)"
		<< "," << "RG_Evap_SV_W_(m3)"
		<< "," << "RG_Evap_SV_SWE_(m3)"
		<< "," << "RG_LeafStore_SV_W_(m3)"
		<< "," << "RG_LeafStore_SV_SWE"
		<< "," << "RG_ThroughFall_SV_W_(m3)"
		<< "," << "RG_ThroughFall_SV_SWE_(m3)"
		<< "," << "RG_ShortVegCanopy_WB_(m3)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				(0) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				(0) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}
		else
		{
			double a = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2);
			double b = ((folder->VarVecDict["treeStoredW"][i - 1] + folder->VarVecDict["treeStoredSWE"][i - 1]) * TotalTreeCover_m2);
			double c = ((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2);
			double e = ((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2);
			double f = ((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);


			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = a + b - c - e - f;
			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				((folder->VarVecDict["shortVegStoredW"][i - 1] + folder->VarVecDict["shortVegStoredSWE"][i - 1]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * TotalTreeCover_m2
			<< "," << input->Snow[i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TotalTreeCover_m2
			<< "," << tree_balance
			<< "," << input->Rain[i] * ShortVegCover_m2
			<< "," << input->Snow[i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_VWB_RainGarden(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_VWB_RainGarden.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;


	double TotalPerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"] + folder->ParamDict["SoilCover_frac"];

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "RG_Evap_TC_W_(m)"
		<< "," << "RG_Evap_TC_SWE_(m)"
		<< "," << "RG_Evap_SV_W_(m)"
		<< "," << "RG_Evap_SV_SWE_(m)"
		<< "," << "RG_Infiltration_PA_(m)"
		<< "," << "RG_rET_VegCover_(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["infilExInfil"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< "," << folder->VarVecDict["rET"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< std::endl;
	}
	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_FWB_RainGarden(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_FWB_RainGarden.csv");
	if (!outfile.good())
		return;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double BareSoilCover_m2 = folder->ParamDict["SoilCover_frac"] *  folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = TotalTreeCover_m2 + ShortVegCover_m2 + BareSoilCover_m2;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "RG_Evap_TC_W_(m3)"
		<< "," << "RG_Evap_TC_SWE_(m3)"
		<< "," << "RG_Evap_SV_W_(m3)"
		<< "," << "RG_Evap_SV_SWE_(m3)"
		<< "," << "RG_Infiltration_PA_(m3)"
		<< "," << "RG_rET_VegCover_(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["infilExInfil"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["rET"][i] * TotalPerviousCover_m2
			<< std::endl;

	}
	outfile.close();
}


void GIOutputWriter::writeVegCanopyFile_VWB_BioRetention(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_VWB_BioRetention.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m)"
		<< "," << "SWE_(m)"
		<< "," << "RG_Intercept_TC_W_(m)"
		<< "," << "RG_Intercept_TC_SWE_(m)"
		<< "," << "RG_Evap_TC_W_(m)"
		<< "," << "RG_Evap_TC_SWE_(m)"
		<< "," << "RG_LeafStore_TC_W_(m)"
		<< "," << "RG_LeafStore_TC_SWE_(m)"
		<< "," << "RG_ThroughFall_TC_W_(m)"
		<< "," << "RG_ThroughFall_TC_SWE_(m)"
		<< "," << "RG_TreeCanopy_WB__(m)"
		<< "," << "RG_Rain_SV_(m)"
		<< "," << "RG_SWE_SV_(m)"
		<< "," << "RG_Intercept_SV_W_(m)"
		<< "," << "RG_Intercept_SW_SWE_(m)"
		<< "," << "RG_Evap_SV_W_(m)"
		<< "," << "RG_Evap_SV_SWE_(m)"
		<< "," << "RG_LeafStore_SV_W_(m)"
		<< "," << "RG_LeafStore_SV_SWE"
		<< "," << "RG_ThroughFall_SV_W_(m)"
		<< "," << "RG_ThroughFall_SV_SWE_(m)"
		<< "," << "RG_ShortVegCanopy_WB_(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["treeStoredW"][i - 1] + folder->VarVecDict["treeStoredSWE"][i - 1]) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["shortVegStoredW"][i - 1] + folder->VarVecDict["shortVegStoredSWE"][i - 1]) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}




		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["treeIntW"][i]
			<< "," << folder->VarVecDict["treeIntSWE"][i]
			<< "," << folder->VarVecDict["treeEvapW"][i]
			<< "," << folder->VarVecDict["treeEvapSWE"][i]
			<< "," << folder->VarVecDict["treeStoredW"][i]
			<< "," << folder->VarVecDict["treeStoredSWE"][i]
			<< "," << folder->VarVecDict["treeThroughW"][i]
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i]
			<< "," << tree_balance
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["shortVegIntW"][i]
			<< "," << folder->VarVecDict["shortVegIntSWE"][i]
			<< "," << folder->VarVecDict["shortVegEvapW"][i]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i]
			<< "," << folder->VarVecDict["shortVegStoredW"][i]
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i]
			<< "," << folder->VarVecDict["shortVegThroughW"][i]
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i]
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeVegCanopyFile_FWB_BioRetention(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_FWB_BioRetention.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m3)"
		<< "," << "SWE_(m3)"
		<< "," << "RG_Rain_TC_(m3)"
		<< "," << "RG_SWE_TC_(m3)"
		<< "," << "RG_Intercept_TC_W_(m3)"
		<< "," << "RG_Intercept_TC_SWE_(m3)"
		<< "," << "RG_Evap_TC_W_(m3)"
		<< "," << "RG_Evap_TC_SWE_(m3)"
		<< "," << "RG_LeafStore_TC_W_(m3)"
		<< "," << "RG_LeafStore_TC_SWE_(m3)"
		<< "," << "RG_ThroughFall_TC_W_(m3)"
		<< "," << "RG_ThroughFall_TC_SWE_(m3)"
		<< "," << "RG_TreeCanopy_WB_(m3)"
		<< "," << "RG_Rain_SV_(m3)"
		<< "," << "RG_SWE_SV_(m3)"
		<< "," << "RG_Intercept_SV_W_(m3)"
		<< "," << "RG_Intercept_SW_SWE_(m3)"
		<< "," << "RG_Evap_SV_W_(m3)"
		<< "," << "RG_Evap_SV_SWE_(m3)"
		<< "," << "RG_LeafStore_SV_W_(m3)"
		<< "," << "RG_LeafStore_SV_SWE"
		<< "," << "RG_ThroughFall_SV_W_(m3)"
		<< "," << "RG_ThroughFall_SV_SWE_(m3)"
		<< "," << "RG_ShortVegCanopy_WB_(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				(0) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				(0) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}
		else
		{
			double a = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2);
			double b = ((folder->VarVecDict["treeStoredW"][i - 1] + folder->VarVecDict["treeStoredSWE"][i - 1]) * TotalTreeCover_m2);
			double c = ((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2);
			double e = ((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2);
			double f = ((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);


			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = a + b - c - e - f;
			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				((folder->VarVecDict["shortVegStoredW"][i - 1] + folder->VarVecDict["shortVegStoredSWE"][i - 1]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * TotalTreeCover_m2
			<< "," << input->Snow[i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TotalTreeCover_m2
			<< "," << tree_balance
			<< "," << input->Rain[i] * ShortVegCover_m2
			<< "," << input->Snow[i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_VWB_BioRetention(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_VWB_BioRetention.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;


	double TotalPerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"] + folder->ParamDict["SoilCover_frac"];

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "RG_Evap_TC_W_(m)"
		<< "," << "RG_Evap_TC_SWE_(m)"
		<< "," << "RG_Evap_SV_W_(m)"
		<< "," << "RG_Evap_SV_SWE_(m)"
		<< "," << "RG_Infiltration_PA_(m)"
		<< "," << "RG_rET_VegCover_(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["infilExInfil"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< "," << folder->VarVecDict["rET"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< std::endl;
	}
	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_FWB_BioRetention(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_FWB_BioRetention.csv");
	if (!outfile.good())
		return;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double BareSoilCover_m2 = folder->ParamDict["SoilCover_frac"] * folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = TotalTreeCover_m2 + ShortVegCover_m2 + BareSoilCover_m2;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "RG_Evap_TC_W_(m3)"
		<< "," << "RG_Evap_TC_SWE_(m3)"
		<< "," << "RG_Evap_SV_W_(m3)"
		<< "," << "RG_Evap_SV_SWE_(m3)"
		<< "," << "RG_Infiltration_PA_(m3)"
		<< "," << "RG_rET_VegCover_(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["infilExInfil"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["rET"][i] * TotalPerviousCover_m2
			<< std::endl;

	}
	outfile.close();
}




void GIOutputWriter::writeVegCanopyFile_VWB_GreenRoof(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_VWB_GreenRoof.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m)"
		<< "," << "SWE_(m)"
		<< "," << "GR_Intercept_TC_W_(m)"
		<< "," << "GR_Intercept_TC_SWE_(m)"
		<< "," << "GR_Evap_TC_W_(m)"
		<< "," << "GR_Evap_TC_SWE_(m)"
		<< "," << "GR_LeafStore_TC_W_(m)"
		<< "," << "GR_LeafStore_TC_SWE_(m)"
		<< "," << "GR_ThroughFall_TC_W_(m)"
		<< "," << "GR_ThroughFall_TC_SWE_(m)"
		<< "," << "GR_TreeCanopy_WB_(m)"
		<< "," << "GR_Rain_SV_(m)"
		<< "," << "GR_SWE_SV_(m)"
		<< "," << "GR_Intercept_SV_W_(m)"
		<< "," << "GR_Intercept_SW_SWE_(m)"
		<< "," << "GR_Evap_SV_W_(m)"
		<< "," << "GR_Evap_SV_SWE_(m)"
		<< "," << "GR_LeafStore_SV_W_(m)"
		<< "," << "GR_LeafStore_SV_SWE"
		<< "," << "GR_ThroughFall_SV_W_(m)"
		<< "," << "GR_ThroughFall_SV_SWE_(m)"
		<< "," << "GR_ShortVegCanopy_WB_(m)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}




		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["treeIntW"][i]
			<< "," << folder->VarVecDict["treeIntSWE"][i]
			<< "," << folder->VarVecDict["treeEvapW"][i]
			<< "," << folder->VarVecDict["treeEvapSWE"][i]
			<< "," << folder->VarVecDict["treeStoredW"][i]
			<< "," << folder->VarVecDict["treeStoredSWE"][i]
			<< "," << folder->VarVecDict["treeThroughW"][i]
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i]
			<< "," << tree_balance
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["shortVegIntW"][i]
			<< "," << folder->VarVecDict["shortVegIntSWE"][i]
			<< "," << folder->VarVecDict["shortVegEvapW"][i]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i]
			<< "," << folder->VarVecDict["shortVegStoredW"][i]
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i]
			<< "," << folder->VarVecDict["shortVegThroughW"][i]
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i]
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeVegCanopyFile_FWB_GreenRoof(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_FWB_GreenRoof.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m3)"
		<< "," << "SWE_(m3)"
		<< "," << "GR_Rain_TC_(m3)"
		<< "," << "GR_SWE_TC_(m3)"
		<< "," << "GR_Intercept_TC_W_(m3)"
		<< "," << "GR_Intercept_TC_SWE_(m3)"
		<< "," << "GR_Evap_TC_W_(m3)"
		<< "," << "GR_Evap_TC_SWE_(m3)"
		<< "," << "GR_LeafStore_TC_W_(m3)"
		<< "," << "GR_LeafStore_TC_SWE_(m3)"
		<< "," << "GR_ThroughFall_TC_W_(m3)"
		<< "," << "GR_ThroughFall_TC_SWE_(m3)"
		<< "," << "GR_TreeCanopy_WB_(m3)"
		<< "," << "GR_Rain_SV_(m3)"
		<< "," << "GR_SWE_SV_(m3)"
		<< "," << "GR_Intercept_SV_W_(m3)"
		<< "," << "GR_Intercept_SW_SWE_(m3)"
		<< "," << "GR_Evap_SV_W_(m3)"
		<< "," << "GR_Evap_SV_SWE_(m3)"
		<< "," << "GR_LeafStore_SV_W_(m3)"
		<< "," << "GR_LeafStore_SV_SWE"
		<< "," << "GR_ThroughFall_SV_W_(m3)"
		<< "," << "GR_ThroughFall_SV_SWE_(m3)"
		<< "," << "GR_ShortVegCanopy_WB_(m3)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				(0) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				(0) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				((folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				((folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * TotalTreeCover_m2
			<< "," << input->Snow[i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TotalTreeCover_m2
			<< "," << tree_balance
			<< "," << input->Rain[i] * ShortVegCover_m2
			<< "," << input->Snow[i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_VWB_GreenRoof(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_VWB_GreenRoof.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double TotalPerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"] + folder->ParamDict["SoilCover_frac"];


	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GR_Evap_TC_W_(m)"
		<< "," << "GR_Evap_TC_SWE_(m)"
		<< "," << "GR_Evap_SV_W_(m)"
		<< "," << "GR_Evap_SV_SWE_(m)"
		<< "," << "GR_Infiltration_PA_(m)"
		<< "," << "GR_rET_VegCover_(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["infilExInfil"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< "," << folder->VarVecDict["rET"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< std::endl;
	}
	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_FWB_GreenRoof(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_FWB_GreenRoof.csv");
	if (!outfile.good())
		return;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double BareSoilCover_m2 = folder->ParamDict["SoilCover_frac"] *  folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = TotalTreeCover_m2 + ShortVegCover_m2 + BareSoilCover_m2;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GR_Evap_TC_W_(m3)"
		<< "," << "GR_Evap_TC_SWE_(m3)"
		<< "," << "GR_Evap_SV_W_(m3)"
		<< "," << "GR_Evap_SV_SWE_(m3)"
		<< "," << "GR_Infiltration_PA_(m3)"
		<< "," << "GR_rET_VegCover_(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["infilExInfil"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["rET"][i] * TotalPerviousCover_m2
			<< std::endl;

	}
	outfile.close();
}


// Changes below copied from pre-refactored semi-distributed code with Rain Barrel outputs added (revision 156 in hydrologyv6 project) by RC, 5/17
//May need modifications
void GIOutputWriter::writeVegCanopyFile_VWB_PermeablePavement(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_VWB_PermeablePavement.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m)"
		<< "," << "SWE_(m)"
		<< "," << "PP_Intercept_TC_W_(m)"
		<< "," << "PP_Intercept_TC_SWE_(m)"
		<< "," << "PP_Evap_TC_W_(m)"
		<< "," << "PP_Evap_TC_SWE_(m)"
		<< "," << "PP_LeafStore_TC_W_(m)"
		<< "," << "PP_LeafStore_TC_SWE_(m)"
		<< "," << "PP_ThroughFall_TC_W_(m)"
		<< "," << "PP_ThroughFall_TC_SWE_(m)"
		<< "," << "PP_TreeCanopy_WB_(m)"
		<< "," << "PP_Rain_SV_(m)"
		<< "," << "PP_SWE_SV_(m)"
		<< "," << "PP_Intercept_SV_W_(m)"
		<< "," << "PP_Intercept_SW_SWE_(m)"
		<< "," << "PP_Evap_SV_W_(m)"
		<< "," << "PP_Evap_SV_SWE_(m)"
		<< "," << "PP_LeafStore_SV_W_(m)"
		<< "," << "PP_LeafStore_SV_SWE"
		<< "," << "PP_ThroughFall_SV_W_(m)"
		<< "," << "PP_ThroughFall_SV_SWE_(m)"
		<< "," << "PP_ShortVegCanopy_WB_(m)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}




		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["treeIntW"][i]
			<< "," << folder->VarVecDict["treeIntSWE"][i]
			<< "," << folder->VarVecDict["treeEvapW"][i]
			<< "," << folder->VarVecDict["treeEvapSWE"][i]
			<< "," << folder->VarVecDict["treeStoredW"][i]
			<< "," << folder->VarVecDict["treeStoredSWE"][i]
			<< "," << folder->VarVecDict["treeThroughW"][i]
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i]
			<< "," << tree_balance
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["shortVegIntW"][i]
			<< "," << folder->VarVecDict["shortVegIntSWE"][i]
			<< "," << folder->VarVecDict["shortVegEvapW"][i]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i]
			<< "," << folder->VarVecDict["shortVegStoredW"][i]
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i]
			<< "," << folder->VarVecDict["shortVegThroughW"][i]
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i]
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}
//May need modifications
void GIOutputWriter::writeVegCanopyFile_FWB_PermeablePavement(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_FWB_PermeablePavement.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m3)"
		<< "," << "SWE_(m3)"
		<< "," << "PP_Rain_TC_(m3)"
		<< "," << "PP_SWE_TC_(m3)"
		<< "," << "PP_Intercept_TC_W_(m3)"
		<< "," << "PP_Intercept_TC_SWE_(m3)"
		<< "," << "PP_Evap_TC_W_(m3)"
		<< "," << "PP_Evap_TC_SWE_(m3)"
		<< "," << "PP_LeafStore_TC_W_(m3)"
		<< "," << "PP_LeafStore_TC_SWE_(m3)"
		<< "," << "PP_ThroughFall_TC_W_(m3)"
		<< "," << "PP_ThroughFall_TC_SWE_(m3)"
		<< "," << "PP_TreeCanopy_WB_(m3)"
		<< "," << "PP_Rain_SV_(m3)"
		<< "," << "PP_SWE_SV_(m3)"
		<< "," << "PP_Intercept_SV_W_(m3)"
		<< "," << "PP_Intercept_SW_SWE_(m3)"
		<< "," << "PP_Evap_SV_W_(m3)"
		<< "," << "PP_Evap_SV_SWE_(m3)"
		<< "," << "PP_LeafStore_SV_W_(m3)"
		<< "," << "PP_LeafStore_SV_SWE"
		<< "," << "PP_ThroughFall_SV_W_(m3)"
		<< "," << "PP_ThroughFall_SV_SWE_(m3)"
		<< "," << "PP_ShortVegCanopy_WB_(m3)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				(0) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				(0) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				((folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				((folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * TotalTreeCover_m2
			<< "," << input->Snow[i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TotalTreeCover_m2
			<< "," << tree_balance
			<< "," << input->Rain[i] * ShortVegCover_m2
			<< "," << input->Snow[i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}
//May need modifications
void GIOutputWriter::writeEvap_ET_Infil_VWB_PermeablePavement(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_VWB_PermeablePavement.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double TotalPerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"] + folder->ParamDict["SoilCover_frac"];


	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "PP_Evap_TC_W_(m)"
		<< "," << "PP_Evap_TC_SWE_(m)"
		<< "," << "PP_Evap_SV_W_(m)"
		<< "," << "PP_Evap_SV_SWE_(m)"
		<< "," << "PP_Infiltration_PA_(m)"
		<< "," << "PP_rET_VegCover_(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		//IDataFolder* folder = organizer->gefolder(3);
		//double value = folder->treeEvapW_vec[0];
		//double value1 = folder->treeEvapSWE_vec[0];
		//double value2 = folder->shortVegEvapW_vec[0];
		//double value3 = folder->shortVegEvapSWE_vec[0];
		//double value4 = folder->infilExInfil_vec[0];
		////Arpit
		////double value5 = folder->rET_vec[0];


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["infilExInfil"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			//Arpit
			//<< "," << folder->VarVecDict["rET"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< std::endl;
	}
	outfile.close();

}
//May need modifications
void GIOutputWriter::writeEvap_ET_Infil_FWB_PermeablePavement(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_FWB_PermeablePavement.csv");
	if (!outfile.good())
		return;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double BareSoilCover_m2 = folder->ParamDict["SoilCover_frac"] *  folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = TotalTreeCover_m2 + ShortVegCover_m2 + BareSoilCover_m2;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "PP_Evap_TC_W_(m3)"
		<< "," << "PP_Evap_TC_SWE_(m3)"
		<< "," << "PP_Evap_SV_W_(m3)"
		<< "," << "PP_Evap_SV_SWE_(m3)"
		<< "," << "PP_Infiltration_PA_(m3)"
		<< "," << "PP_rET_VegCover_(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["infilExInfil"][i] * TotalPerviousCover_m2
			//Arpit
			//<< "," << folder->VarVecDict["rET"][i] * TotalPerviousCover_m2
			<< std::endl;

	}
	outfile.close();
}

//May need modifications
void GIOutputWriter::writeFlow_VWB_RainBarrel(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Flow_VWB_RainBarrel.csv");
	if (!outfile.good())
		return;
	// Disabling as Rain Barrel folders will have no veg cover - RC, 04/17
	/*double tree_balance = 0.0;
	double shortveg_balance = 0.0;*/


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m)"
		<< "," << "SWE_(m)"
		<< "," << "RBArea_Precip_(m)"
		<< "," << "RB_Drainage_(m)"
		<< "," << "RB_Storage_(m)"
		<< "," << "RB_StorageMax_(m)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		//Disabling as Rain Barrel folders will have no veg cover - RC, 04/17
		//if (i == 0)
		//{
		//	// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
		//	tree_balance = (input->Rain[i] + input->Snow[i]) +
		//		(0) -
		//		(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
		//		(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
		//		(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

		//	shortveg_balance = (input->Rain[i] + input->Snow[i]) +
		//		(0) -
		//		(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
		//		(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
		//		(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		//}
		//else
		//{
		//	// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
		//	tree_balance = (input->Rain[i] + input->Snow[i]) +
		//		(folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) -
		//		(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
		//		(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
		//		(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

		//	shortveg_balance = (input->Rain[i] + input->Snow[i]) +
		//		(folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) -
		//		(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
		//		(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
		//		(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		//}

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["waterOnRainBarrelArea"][i] / folder->ParamDict["Area"] // RBArea_Precip in 1 dimension
			<< "," << folder->VarVecDict["rainBarrelFlowToInfil"][i] // RB_Drainage in 1d and per all barrels
			<< "," << (folder->VarVecDict["rbStorage"][i] * folder->ParamDict["RainBarrelNum"]) / folder->ParamDict["Area"]
			<< "," << (folder->ParamDict["RainBarrelVolume"] * folder->ParamDict["RainBarrelNum"]) / folder->ParamDict["Area"]
			<< endl;
	}

	outfile.close();

}
//May need modifications
void GIOutputWriter::writeFlow_FWB_RainBarrel(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Flow_FWB_RainBarrel.csv");
	if (!outfile.good())
		return;
	// Disabling as Rain Barrel folders will have no veg cover - RC, 04/17
	//double tree_balance = 0.0;
	//double shortveg_balance = 0.0;

	//double TotalTreeCover_m2 = 1; //folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	//double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m3)"
		<< "," << "SWE_(m3)"
		<< "," << "RBArea_Precip_(m3)"
		<< "," << "RB_Drainage_(m3)"
		<< "," << "RB_Storage_(m3)"
		<< "," << "RB_StorageMax_(m3)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		//Disabling as Rain Barrel folders will have no veg cover - RC, 04/17
		//if (i == 0)
		//{
		//	// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
		//	tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
		//		(0) -
		//		((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
		//		((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
		//		((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

		//	shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
		//		(0) -
		//		((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
		//		((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
		//		((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		//}
		//else
		//{
		//	// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
		//	tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
		//		((folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) * TotalTreeCover_m2) -
		//		((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
		//		((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
		//		((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

		//	shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
		//		((folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) * ShortVegCover_m2) -
		//		((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
		//		((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
		//		((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		//}

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"] * folder->ParamDict["RainBarrelNum"]
			<< "," << input->Snow[i] * folder->ParamDict["Area"] * folder->ParamDict["RainBarrelNum"]
			<< "," << folder->VarVecDict["waterOnRainBarrelArea"][i] * folder->ParamDict["RainBarrelNum"]
			<< "," << folder->VarVecDict["rainBarrelFlowToInfil"][i] * folder->ParamDict["Area"]
			<< "," << folder->VarVecDict["rbStorage"][i] * folder->ParamDict["RainBarrelNum"]
			<< "," << folder->ParamDict["RainBarrelVolume"] * folder->ParamDict["RainBarrelNum"]
			<< endl;
	}

	outfile.close();

}
// Commenting out and likely to delete WaterInRootZone_..._RainBarrel files 
// cuz these outputs are not relevant and the output writer is crashing midway through this code - RC 4/17
//Uncommented and modified by Li
////May need modifications
void GIOutputWriter::writeWaterInRootZone_VWB_RainBarrel(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterInRootZone_VWB_RainBarrel.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double RootZone_WB = 0.0;
	double perAreaRunoff_WB = 0.0;
	double Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "RB_PA_Flow_To_Infil_(m)"
		<< "," << "RB_Infil_InfilExcess_(m)"
		<< "," << "RB_Infil_SatExcess_(m)"
		<< "," << "RB_Infil_Weighted_Total_(m)"
		<< "," << "RB_Infil_WB_(m)"
		<< "," << "RB_Runoff_InfilExcess_(m)"
		<< "," << "RB_Runoff_SatExcess_(m)"
		<< "," << "RB_RootZone_WB_(m)"
		<< "," << "RB_Total_PA_Runoff_(m)"
		<< "," << "RB_PA_Runoff_WB(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		Infil_WB = folder->VarVecDict["infilWeighted"][i] -
			folder->VarVecDict["satExInfil"][i] -
			folder->VarVecDict["adjInfilExInfil"][i];


		RootZone_WB = folder->VarVecDict["runoffToInfil_postPMacro"][i] -
			(folder->VarVecDict["infilWeighted"][i] - folder->VarVecDict["satExRunoff"][i]) -
			(folder->VarVecDict["satExRunoff"][i] /** fracSatExcess*/) -
			(folder->VarVecDict["adjInfilExRunoff"][i]);

		perAreaRunoff_WB = (folder->VarVecDict["satExRunoff"][i] /** fracSatExcess*/) +
			(folder->VarVecDict["adjInfilExRunoff"][i] /** fracInfilExcess*/) -
			folder->VarVecDict["perAreaFlow"][i];

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["runoffToInfil_postPMacro"][i]
			<< "," << folder->VarVecDict["adjInfilExInfil"][i]										// already weighted
			<< "," << folder->VarVecDict["satExInfil"][i]										// already weighted
			<< "," << folder->VarVecDict["infilExInfil"][i]								// already weighted
			<< "," << Infil_WB
			<< "," << folder->VarVecDict["adjInfilExRunoff"][i] //* fracInfilExcess
			<< "," << folder->VarVecDict["satExRunoff"][i] //* fracSatExcess    All of the soil contributes to saturation excess after the inital inflow constraint
			<< "," << RootZone_WB
			<< "," << folder->VarVecDict["perAreaFlow_unAdj"][i]
			<< "," << perAreaRunoff_WB
			<< std::endl;

	}
	outfile.close();

}
////May need modifications
void GIOutputWriter::writeWaterInRootZone_FWB_RainBarrel(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterInRootZone_FWB_RainBarrel.csv");
	if (!outfile.good())
		return;

	double TotalPerviousCover_m2 = folder->ParamDict["TotalPerviousCover_frac"] * folder->ParamDict["Area"];

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double RootZone_WB = 0.0;
	double perAreaRunoff_WB = 0.0;
	double Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "RB_PA_Flow_To_Infil_(m3)"
		<< "," << "RB_Infil_InfilExcess_(m3)"
		<< "," << "RB_Infil_SatExcess_(m3)"
		<< "," << "RB_Infil_Weighted_Total_(m3)"
		<< "," << "RB_Infil_WB_(m3)"
		<< "," << "RB_Runoff_InfilExcess_(m3)"
		<< "," << "RB_Runoff_SatExcess_(m3)"
		<< "," << "RB_RootZone_WB_(m3)"
		<< "," << "RB_Total_PA_Runoff_(m3)"
		<< "," << "RB_PA_Runoff_WB(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		Infil_WB = (folder->VarVecDict["infilWeighted"][i] * TotalPerviousCover_m2) -
			(folder->VarVecDict["satExInfil"][i] * TotalPerviousCover_m2) -
			(folder->VarVecDict["adjInfilExInfil"][i] * TotalPerviousCover_m2);


		RootZone_WB = (folder->VarVecDict["runoffToInfil_postPMacro"][i] * TotalPerviousCover_m2) -
			((folder->VarVecDict["infilWeighted"][i] - folder->VarVecDict["satExRunoff"][i])* TotalPerviousCover_m2) -
			(folder->VarVecDict["satExRunoff"][i] * TotalPerviousCover_m2  /** fracSatExcess*/) -
			(folder->VarVecDict["adjInfilExRunoff"][i] * TotalPerviousCover_m2);

		perAreaRunoff_WB = (folder->VarVecDict["satExRunoff"][i] * TotalPerviousCover_m2 /** fracSatExcess*/) +
			(folder->VarVecDict["adjInfilExRunoff"][i] * TotalPerviousCover_m2 /** fracInfilExcess*/) -
			(folder->VarVecDict["perAreaFlow"][i] * TotalPerviousCover_m2);

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["runoffToInfil_postPMacro"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["adjInfilExInfil"][i] * TotalPerviousCover_m2										// already weighted
			<< "," << folder->VarVecDict["satExInfil"][i] * TotalPerviousCover_m2										// already weighted
			<< "," << folder->VarVecDict["infilExInfil"][i] * TotalPerviousCover_m2								// already weighted
			<< "," << Infil_WB
			<< "," << folder->VarVecDict["adjInfilExRunoff"][i] * TotalPerviousCover_m2 //* fracInfilExcess
			<< "," << folder->VarVecDict["satExRunoff"][i] * TotalPerviousCover_m2 //* fracSatExcess    All of the soil contributes to saturation excess after the inital inflow constraint
			<< "," << RootZone_WB
			<< "," << folder->VarVecDict["perAreaFlow_unAdj"][i] * TotalPerviousCover_m2
			<< "," << perAreaRunoff_WB
			<< std::endl;

	}
	outfile.close();
}



void GIOutputWriter::writeVegCanopyFile_VWB_TreePit(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_VWB_TreePit.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m)"
		<< "," << "SWE_(m)"
		<< "," << "TP_Intercept_TC_W_(m)"
		<< "," << "TP_Intercept_TC_SWE_(m)"
		<< "," << "TP_Evap_TC_W_(m)"
		<< "," << "TP_Evap_TC_SWE_(m)"
		<< "," << "TP_LeafStore_TC_W_(m)"
		<< "," << "TP_LeafStore_TC_SWE_(m)"
		<< "," << "TP_ThroughFall_TC_W_(m)"
		<< "," << "TP_ThroughFall_TC_SWE_(m)"
		<< "," << "TP_TreeCanopy_WB_(m)"
		<< "," << "TP_Rain_SV_(m)"
		<< "," << "TP_SWE_SV_(m)"
		<< "," << "TP_Intercept_SV_W_(m)"
		<< "," << "TP_Intercept_SW_SWE_(m)"
		<< "," << "TP_Evap_SV_W_(m)"
		<< "," << "TP_Evap_SV_SWE_(m)"
		<< "," << "TP_LeafStore_SV_W_(m)"
		<< "," << "TP_LeafStore_SV_SWE"
		<< "," << "TP_ThroughFall_SV_W_(m)"
		<< "," << "TP_ThroughFall_SV_SWE_(m)"
		<< "," << "TP_ShortVegCanopy_WB_(m)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}




		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["treeIntW"][i]
			<< "," << folder->VarVecDict["treeIntSWE"][i]
			<< "," << folder->VarVecDict["treeEvapW"][i]
			<< "," << folder->VarVecDict["treeEvapSWE"][i]
			<< "," << folder->VarVecDict["treeStoredW"][i]
			<< "," << folder->VarVecDict["treeStoredSWE"][i]
			<< "," << folder->VarVecDict["treeThroughW"][i]
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i]
			<< "," << tree_balance
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["shortVegIntW"][i]
			<< "," << folder->VarVecDict["shortVegIntSWE"][i]
			<< "," << folder->VarVecDict["shortVegEvapW"][i]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i]
			<< "," << folder->VarVecDict["shortVegStoredW"][i]
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i]
			<< "," << folder->VarVecDict["shortVegThroughW"][i]
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i]
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeVegCanopyFile_FWB_TreePit(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_FWB_TreePit.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m3)"
		<< "," << "SWE_(m3)"
		<< "," << "TP_Rain_TC_(m3)"
		<< "," << "TP_SWE_TC_(m3)"
		<< "," << "TP_Intercept_TC_W_(m3)"
		<< "," << "TP_Intercept_TC_SWE_(m3)"
		<< "," << "TP_Evap_TC_W_(m3)"
		<< "," << "TP_Evap_TC_SWE_(m3)"
		<< "," << "TP_LeafStore_TC_W_(m3)"
		<< "," << "TP_LeafStore_TC_SWE_(m3)"
		<< "," << "TP_ThroughFall_TC_W_(m3)"
		<< "," << "TP_ThroughFall_TC_SWE_(m3)"
		<< "," << "TP_TreeCanopy_WB_(m3)"
		<< "," << "TP_Rain_SV_(m3)"
		<< "," << "TP_SWE_SV_(m3)"
		<< "," << "TP_Intercept_SV_W_(m3)"
		<< "," << "TP_Intercept_SW_SWE_(m3)"
		<< "," << "TP_Evap_SV_W_(m3)"
		<< "," << "TP_Evap_SV_SWE_(m3)"
		<< "," << "TP_LeafStore_SV_W_(m3)"
		<< "," << "TP_LeafStore_SV_SWE"
		<< "," << "TP_ThroughFall_SV_W_(m3)"
		<< "," << "TP_ThroughFall_SV_SWE_(m3)"
		<< "," << "TP_ShortVegCanopy_WB_(m3)"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				(0) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				(0) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				((folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				((folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * TotalTreeCover_m2
			<< "," << input->Snow[i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TotalTreeCover_m2
			<< "," << tree_balance
			<< "," << input->Rain[i] * ShortVegCover_m2
			<< "," << input->Snow[i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_VWB_TreePit(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_VWB_TreePit.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double TotalPerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"] + folder->ParamDict["SoilCover_frac"];


	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "TP_Evap_TC_W_(m)"
		<< "," << "TP_Evap_TC_SWE_(m)"
		<< "," << "TP_Evap_SV_W_(m)"
		<< "," << "TP_Evap_SV_SWE_(m)"
		<< "," << "TP_Infiltration_PA_(m)"
		<< "," << "TP_rET_VegCover_(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
	/*	IDataFolder* folder = organizer->gefolder(5);
		double value = folder->treeEvapW_vec[0];
		double value1 = folder->treeEvapSWE_vec[0];
		double value2 = folder->shortVegEvapW_vec[0];
		double value3 = folder->shortVegEvapSWE_vec[0];
		double value4 = folder->infilExInfil_vec[0];
		double value5 = folder->rET_vec[0];
*/

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["infilExInfil"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< "," << folder->VarVecDict["rET"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< std::endl;
	}
	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_FWB_TreePit(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_FWB_TreePit.csv");
	if (!outfile.good())
		return;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double BareSoilCover_m2 = folder->ParamDict["SoilCover_frac"] *  folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = TotalTreeCover_m2 + ShortVegCover_m2 + BareSoilCover_m2;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "TP_Evap_TC_W_(m3)"
		<< "," << "TP_Evap_TC_SWE_(m3)"
		<< "," << "TP_Evap_SV_W_(m3)"
		<< "," << "TP_Evap_SV_SWE_(m3)"
		<< "," << "TP_Infiltration_PA_(m3)"
		<< "," << "TP_rET_VegCover_(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["infilExInfil"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["rET"][i] * TotalPerviousCover_m2
			<< std::endl;

	}
	outfile.close();
}

void GIOutputWriter::writeWaterInRootZone_VWB_TreePit(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterInRootZone_VWB_TreePit.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double RootZone_WB = 0.0;
	double perAreaRunoff_WB = 0.0;
	double Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "TP_PA_Flow_To_Infil_(m)"
		<< "," << "TP_Infil_InfilExcess_(m)"
		<< "," << "TP_Infil_SatExcess_(m)"
		<< "," << "TP_Infil_Weighted_Total_(m)"
		<< "," << "TP_Infil_WB_(m)"
		<< "," << "TP_Runoff_InfilExcess_(m)"
		<< "," << "TP_Runoff_SatExcess_(m)"
		<< "," << "TP_RootZone_WB_(m)"
		<< "," << "TP_Total_PA_Runoff_(m)"
		<< "," << "TP_PA_Runoff_WB(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		Infil_WB = folder->VarVecDict["infilWeighted"][i] -
			folder->VarVecDict["satExInfil"][i] -
			folder->VarVecDict["adjInfilExInfil"][i];


		RootZone_WB = folder->VarVecDict["runoffToInfil_postPMacro"][i] -
			(folder->VarVecDict["infilWeighted"][i] - folder->VarVecDict["satExRunoff"][i]) -
			(folder->VarVecDict["satExRunoff"][i] /** fracSatExcess*/) -
			(folder->VarVecDict["adjInfilExRunoff"][i]);

		perAreaRunoff_WB = (folder->VarVecDict["satExRunoff"][i] /** fracSatExcess*/) +
			(folder->VarVecDict["adjInfilExRunoff"][i] /** fracInfilExcess*/) -
			folder->VarVecDict["perAreaFlow"][i];

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["runoffToInfil_postPMacro"][i]
			<< "," << folder->VarVecDict["adjInfilExInfil"][i]										// already weighted
			<< "," << folder->VarVecDict["satExInfil"][i]										// already weighted
			<< "," << folder->VarVecDict["infilExInfil"][i]								// already weighted
			<< "," << Infil_WB
			<< "," << folder->VarVecDict["adjInfilExRunoff"][i] //* fracInfilExcess
			<< "," << folder->VarVecDict["satExRunoff"][i] //* fracSatExcess    All of the soil contributes to saturation excess after the inital inflow constraint
			<< "," << RootZone_WB
			<< "," << folder->VarVecDict["perAreaFlow_unAdj"][i]
			<< "," << perAreaRunoff_WB
			<< std::endl;

	}
	outfile.close();

}

void GIOutputWriter::writeWaterInRootZone_FWB_TreePit(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterInRootZone_FWB_TreePit.csv");
	if (!outfile.good())
		return;

	double TotalPerviousCover_m2 = folder->ParamDict["TotalPerviousCover_frac"] * folder->ParamDict["Area"];

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double RootZone_WB = 0.0;
	double perAreaRunoff_WB = 0.0;
	double Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "TP_PA_Flow_To_Infil_(m3)"
		<< "," << "TP_Infil_InfilExcess_(m3)"
		<< "," << "TP_Infil_SatExcess_(m3)"
		<< "," << "TP_Infil_Weighted_Total_(m3)"
		<< "," << "TP_Infil_WB_(m3)"
		<< "," << "TP_Runoff_InfilExcess_(m3)"
		<< "," << "TP_Runoff_SatExcess_(m3)"
		<< "," << "TP_RootZone_WB_(m3)"
		<< "," << "TP_Total_PA_Runoff_(m3)"
		<< "," << "TP_PA_Runoff_WB(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		Infil_WB = (folder->VarVecDict["infilWeighted"][i] * TotalPerviousCover_m2) -
			(folder->VarVecDict["satExInfil"][i] * TotalPerviousCover_m2) -
			(folder->VarVecDict["adjInfilExInfil"][i] * TotalPerviousCover_m2);


		RootZone_WB = (folder->VarVecDict["runoffToInfil_postPMacro"][i] * TotalPerviousCover_m2) -
			((folder->VarVecDict["infilWeighted"][i] - folder->VarVecDict["satExRunoff"][i])* TotalPerviousCover_m2) -
			(folder->VarVecDict["satExRunoff"][i] * TotalPerviousCover_m2  /** fracSatExcess*/) -
			(folder->VarVecDict["adjInfilExRunoff"][i] * TotalPerviousCover_m2);

		perAreaRunoff_WB = (folder->VarVecDict["satExRunoff"][i] * TotalPerviousCover_m2 /** fracSatExcess*/) +
			(folder->VarVecDict["adjInfilExRunoff"][i] * TotalPerviousCover_m2 /** fracInfilExcess*/) -
			(folder->VarVecDict["perAreaFlow"][i] * TotalPerviousCover_m2);

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["runoffToInfil_postPMacro"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["adjInfilExInfil"][i] * TotalPerviousCover_m2										// already weighted
			<< "," << folder->VarVecDict["satExInfil"][i] * TotalPerviousCover_m2										// already weighted
			<< "," << folder->VarVecDict["infilExInfil"][i] * TotalPerviousCover_m2								// already weighted
			<< "," << Infil_WB
			<< "," << folder->VarVecDict["adjInfilExRunoff"][i] * TotalPerviousCover_m2 //* fracInfilExcess
			<< "," << folder->VarVecDict["satExRunoff"][i] * TotalPerviousCover_m2 //* fracSatExcess    All of the soil contributes to saturation excess after the inital inflow constraint
			<< "," << RootZone_WB
			<< "," << folder->VarVecDict["perAreaFlow_unAdj"][i] * TotalPerviousCover_m2
			<< "," << perAreaRunoff_WB
			<< std::endl;

	}
	outfile.close();
}


//For RoofDiscon, added by Li
void GIOutputWriter::writeVegCanopyFile_VWB_RoofDiscon(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_VWB_RoofDiscon.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m)"
		<< "," << "SWE_(m)"
		<< "," << "PP_Intercept_TC_W_(m)"
		<< "," << "PP_Intercept_TC_SWE_(m)"
		<< "," << "PP_Evap_TC_W_(m)"
		<< "," << "PP_Evap_TC_SWE_(m)"
		<< "," << "PP_LeafStore_TC_W_(m)"
		<< "," << "PP_LeafStore_TC_SWE_(m)"
		<< "," << "PP_ThroughFall_TC_W_(m)"
		<< "," << "PP_ThroughFall_TC_SWE_(m)"
		<< "," << "PP_TreeCanopy_WB_(m)"
		<< "," << "PP_Rain_SV_(m)"
		<< "," << "PP_SWE_SV_(m)"
		<< "," << "PP_Intercept_SV_W_(m)"
		<< "," << "PP_Intercept_SW_SWE_(m)"
		<< "," << "PP_Evap_SV_W_(m)"
		<< "," << "PP_Evap_SV_SWE_(m)"
		<< "," << "PP_LeafStore_SV_W_(m)"
		<< "," << "PP_LeafStore_SV_SWE"
		<< "," << "PP_ThroughFall_SV_W_(m)"
		<< "," << "PP_ThroughFall_SV_SWE_(m)"
		<< "," << "PP_ShortVegCanopy_WB_(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["treeStoredW"][i - 1] + folder->VarVecDict["treeStoredSWE"][i - 1]) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["shortVegStoredW"][i - 1] + folder->VarVecDict["shortVegStoredSWE"][i - 1]) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}




		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["treeIntW"][i]
			<< "," << folder->VarVecDict["treeIntSWE"][i]
			<< "," << folder->VarVecDict["treeEvapW"][i]
			<< "," << folder->VarVecDict["treeEvapSWE"][i]
			<< "," << folder->VarVecDict["treeStoredW"][i]
			<< "," << folder->VarVecDict["treeStoredSWE"][i]
			<< "," << folder->VarVecDict["treeThroughW"][i]
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i]
			<< "," << tree_balance
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["shortVegIntW"][i]
			<< "," << folder->VarVecDict["shortVegIntSWE"][i]
			<< "," << folder->VarVecDict["shortVegEvapW"][i]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i]
			<< "," << folder->VarVecDict["shortVegStoredW"][i]
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i]
			<< "," << folder->VarVecDict["shortVegThroughW"][i]
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i]
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}
//May need modifications
void GIOutputWriter::writeVegCanopyFile_FWB_RoofDiscon(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_FWB_RoofDiscon.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_(m3)"
		<< "," << "SWE_(m3)"
		<< "," << "PP_Rain_TC_(m3)"
		<< "," << "PP_SWE_TC_(m3)"
		<< "," << "PP_Intercept_TC_W_(m3)"
		<< "," << "PP_Intercept_TC_SWE_(m3)"
		<< "," << "PP_Evap_TC_W_(m3)"
		<< "," << "PP_Evap_TC_SWE_(m3)"
		<< "," << "PP_LeafStore_TC_W_(m3)"
		<< "," << "PP_LeafStore_TC_SWE_(m3)"
		<< "," << "PP_ThroughFall_TC_W_(m3)"
		<< "," << "PP_ThroughFall_TC_SWE_(m3)"
		<< "," << "PP_TreeCanopy_WB_(m3)"
		<< "," << "PP_Rain_SV_(m3)"
		<< "," << "PP_SWE_SV_(m3)"
		<< "," << "PP_Intercept_SV_W_(m3)"
		<< "," << "PP_Intercept_SW_SWE_(m3)"
		<< "," << "PP_Evap_SV_W_(m3)"
		<< "," << "PP_Evap_SV_SWE_(m3)"
		<< "," << "PP_LeafStore_SV_W_(m3)"
		<< "," << "PP_LeafStore_SV_SWE"
		<< "," << "PP_ThroughFall_SV_W_(m3)"
		<< "," << "PP_ThroughFall_SV_SWE_(m3)"
		<< "," << "PP_ShortVegCanopy_WB_(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				(0) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				(0) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				((folder->VarVecDict["treeStoredW"][i - 1] + folder->VarVecDict["treeStoredSWE"][i - 1]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				((folder->VarVecDict["shortVegStoredW"][i - 1] + folder->VarVecDict["shortVegStoredSWE"][i - 1]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * TotalTreeCover_m2
			<< "," << input->Snow[i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TotalTreeCover_m2
			<< "," << tree_balance
			<< "," << input->Rain[i] * ShortVegCover_m2
			<< "," << input->Snow[i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}
//May need modifications
void GIOutputWriter::writeEvap_ET_Infil_VWB_RoofDiscon(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_VWB_RoofDiscon.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double TotalPerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"] + folder->ParamDict["SoilCover_frac"];


	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "PP_Evap_TC_W_(m)"
		<< "," << "PP_Evap_TC_SWE_(m)"
		<< "," << "PP_Evap_SV_W_(m)"
		<< "," << "PP_Evap_SV_SWE_(m)"
		<< "," << "PP_Infiltration_PA_(m)"
		<< "," << "PP_rET_VegCover_(m)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		//IDataFolder* folder = organizer->gefolder(3);
		//double value = folder->treeEvapW_vec[0];
		//double value1 = folder->treeEvapSWE_vec[0];
		//double value2 = folder->shortVegEvapW_vec[0];
		//double value3 = folder->shortVegEvapSWE_vec[0];
		//double value4 = folder->infilExInfil_vec[0];
		////Arpit
		////double value5 = folder->rET_vec[0];


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["infilExInfil"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			//Arpit
			//<< "," << folder->VarVecDict["rET"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< std::endl;
	}
	outfile.close();

}
//May need modifications
void GIOutputWriter::writeEvap_ET_Infil_FWB_RoofDiscon(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_FWB_RoofDiscon.csv");
	if (!outfile.good())
		return;

	double TotalTreeCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double BareSoilCover_m2 = folder->ParamDict["SoilCover_frac"] * folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = TotalTreeCover_m2 + ShortVegCover_m2 + BareSoilCover_m2;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "PP_Evap_TC_W_(m3)"
		<< "," << "PP_Evap_TC_SWE_(m3)"
		<< "," << "PP_Evap_SV_W_(m3)"
		<< "," << "PP_Evap_SV_SWE_(m3)"
		<< "," << "PP_Infiltration_PA_(m3)"
		<< "," << "PP_rET_VegCover_(m3)"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["infilExInfil"][i] * TotalPerviousCover_m2
			//Arpit
			//<< "," << folder->VarVecDict["rET"][i] * TotalPerviousCover_m2
			<< std::endl;

	}
	outfile.close();
}

